@@ -17,18 +17,15 @@ points -- are excluded here, and left to the EnzoDerivedFields.)
 
 import numpy as np
 
-from yt.funcs import *
-
-from yt.config import ytcfg
-from yt.units.yt_array import YTArray, uconcatenate, array_like_field
-from yt.utilities.exceptions import YTFieldNotFound
+from yt.funcs import \
+    camelcase_to_underscore, \
+    ensure_list
+from yt.units.yt_array import array_like_field
 from yt.utilities.parallel_tools.parallel_analysis_interface import \
     ParallelAnalysisInterface, parallel_objects
-from yt.utilities.lib.Octree import Octree
 from yt.utilities.physical_constants import \
-    gravitational_constant_cgs, \
-    HUGE
-from yt.utilities.math_utils import prec_accum
+    gravitational_constant_cgs
+from yt.utilities.physical_ratios import HUGE
 from yt.extern.six import add_metaclass
 
 derived_quantity_registry = {}
@@ -61,6 +58,8 @@ class DerivedQuantity(ParallelAnalysisInterface):
 
     def __call__(self, *args, **kwargs):
         """Calculate results for the derived quantity"""
+        # create the index if it doesn't exist yet
+        self.data_source.ds.index
         self.count_values(*args, **kwargs)
         chunks = self.data_source.chunks([], chunking_style="io")
         storage = {}
@@ -115,9 +114,10 @@ class WeightedAverageQuantity(DerivedQuantity):
 
     Parameters
     ----------
-    fields : field or list of fields
+
+    fields : string / tuple, or list of strings / tuples
         The field or fields of which the average value is to be calculated.
-    weight : field
+    weight : string or tuple
         The weight field.
 
     Examples
@@ -202,7 +202,6 @@ class TotalMass(TotalQuantity):
     def __call__(self):
         self.data_source.ds.index
         fi = self.data_source.ds.field_info
-        fields = []
         if ("gas", "cell_mass") in fi:
             gas = super(TotalMass, self).__call__([('gas', 'cell_mass')])
         else:
@@ -256,7 +255,8 @@ class CenterOfMass(DerivedQuantity):
           (("all", "particle_mass") in self.data_source.ds.field_info)
         vals = []
         if use_gas:
-            vals += [(data[ax] * data["gas", "cell_mass"]).sum(dtype=np.float64)
+            vals += [(data["gas", ax] *
+                      data["gas", "cell_mass"]).sum(dtype=np.float64)
                      for ax in 'xyz']
             vals.append(data["gas", "cell_mass"].sum(dtype=np.float64))
         if use_particles:
@@ -359,10 +359,10 @@ class WeightedVariance(DerivedQuantity):
 
     Parameters
     ----------
-    fields : field or list of fields
-        The field or fields of which the variance and mean values are
-        to be calculated.
-    weight : field
+
+    fields : string / tuple, or list of strings / tuples
+        The field or fields of which the average value is to be calculated.
+    weight : string or tuple
         The weight field.
 
     Examples
@@ -439,22 +439,26 @@ class AngularMomentumVector(DerivedQuantity):
     """
     def count_values(self, use_gas=True, use_particles=True):
         num_vals = 0
-        if use_gas: num_vals += 4
-        if use_particles: num_vals += 4
+        # create the index if it doesn't exist yet
+        self.data_source.ds.index
+        self.use_gas = use_gas & \
+            (("gas", "cell_mass") in self.data_source.ds.field_info)
+        self.use_particles = use_particles & \
+            (("all", "particle_mass") in self.data_source.ds.field_info)
+        if self.use_gas:
+            num_vals += 4
+        if self.use_particles:
+            num_vals += 4
         self.num_vals = num_vals
 
-    def process_chunk(self, data, use_gas=True, use_particles=True):
-        use_gas &= \
-          (("gas", "cell_mass") in self.data_source.ds.field_info)
-        use_particles &= \
-          (("all", "particle_mass") in self.data_source.ds.field_info)
+    def process_chunk(self, data, **kwargs):
         rvals = []
-        if use_gas:
+        if self.use_gas:
             rvals.extend([(data["gas", "specific_angular_momentum_%s" % axis] *
                            data["gas", "cell_mass"]).sum(dtype=np.float64) \
                           for axis in "xyz"])
             rvals.append(data["gas", "cell_mass"].sum(dtype=np.float64))
-        if use_particles:
+        if self.use_particles:
             rvals.extend([(data["all", "particle_specific_angular_momentum_%s" % axis] *
                            data["all", "particle_mass"]).sum(dtype=np.float64) \
                           for axis in "xyz"])
@@ -525,14 +529,62 @@ class Extrema(DerivedQuantity):
         return [self.data_source.ds.arr([mis.min(), mas.max()])
                 for mis, mas in zip(values[::2], values[1::2])]
 
-class MaxLocation(DerivedQuantity):
+class SampleAtMaxFieldValues(DerivedQuantity):
     r"""
-    Calculates the maximum value plus the index, x, y, and z position
-    of the maximum.
+    Calculates the maximum value and returns whichever fields are asked to be
+    sampled.
 
     Parameters
     ----------
-    field : field
+    field : tuple or string
+        The field over which the extrema are to be calculated.
+    sample_fields : list of fields
+        The fields to sample and return at the minimum value.
+
+    Examples
+    --------
+
+    >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
+    >>> ad = ds.all_data()
+    >>> print ad.quantities.sample_at_max_field_values(("gas", "density"),
+    ...         ["temperature", "velocity_magnitude"])
+
+    """
+    def count_values(self, field, sample_fields):
+        # field itself, then index, then the number of sample fields
+        self.num_vals = 1 + len(sample_fields)
+
+    def __call__(self, field, sample_fields):
+        rv = super(SampleAtMaxFieldValues, self).__call__(field, sample_fields)
+        if len(rv) == 1: rv = rv[0]
+        return rv
+
+    def process_chunk(self, data, field, sample_fields):
+        field = data._determine_fields(field)[0]
+        ma = array_like_field(data, -HUGE, field)
+        vals = [array_like_field(data, -1, sf) for sf in sample_fields]
+        maxi = -1
+        if data[field].size > 0:
+            maxi = self._func(data[field])
+            ma = data[field][maxi]
+            vals = [data[sf][maxi] for sf in sample_fields]
+        return (ma,) + tuple(vals)
+
+    def reduce_intermediate(self, values):
+        i = self._func(values[0]) # ma is values[0]
+        return [val[i] for val in values]
+
+    def _func(self, arr):
+        return np.argmax(arr)
+
+class MaxLocation(SampleAtMaxFieldValues):
+    r"""
+    Calculates the maximum value plus the x, y, and z position of the maximum.
+
+    Parameters
+    ----------
+
+    field : tuple or string
         The field over which the extrema are to be calculated.
 
     Examples
@@ -543,41 +595,46 @@ class MaxLocation(DerivedQuantity):
     >>> print ad.quantities.max_location(("gas", "density"))
 
     """
-    def count_values(self, *args, **kwargs):
-        self.num_vals = 5
-
     def __call__(self, field):
-        rv = super(MaxLocation, self).__call__(field)
+        # Make sure we have an index
+        self.data_source.index
+        sample_fields = get_position_fields(field, self.data_source)
+        rv = super(MaxLocation, self).__call__(field, sample_fields)
         if len(rv) == 1: rv = rv[0]
         return rv
 
-    def process_chunk(self, data, field):
-        axis_names = data.ds.coordinates.axis_name
-        field = data._determine_fields(field)[0]
-        ma = array_like_field(data, -HUGE, field)
-        position_fields = get_position_fields(field, data)
-        mx = array_like_field(data, -1, position_fields[0])
-        my = array_like_field(data, -1, position_fields[1])
-        mz = array_like_field(data, -1, position_fields[2])
-        maxi = -1
-        if data[field].size > 0:
-            maxi = np.argmax(data[field])
-            ma = data[field][maxi]
-            mx, my, mz = [data[ax][maxi] for ax in position_fields]
-        return (ma, maxi, mx, my, mz)
-
-    def reduce_intermediate(self, values):
-        i = np.argmax(values[0]) # ma is values[0]
-        return [val[i] for val in values]
-
-class MinLocation(DerivedQuantity):
+class SampleAtMinFieldValues(SampleAtMaxFieldValues):
     r"""
-    Calculates the minimum value plus the index, x, y, and z position
-    of the minimum.
+    Calculates the minimum value and returns whichever fields are asked to be
+    sampled.
 
     Parameters
     ----------
-    field : field
+    field : tuple or string
+        The field over which the extrema are to be calculated.
+    sample_fields : list of fields
+        The fields to sample and return at the minimum value.
+
+    Examples
+    --------
+
+    >>> ds = load("IsolatedGalaxy/galaxy0030/galaxy0030")
+    >>> ad = ds.all_data()
+    >>> print ad.quantities.sample_at_min_field_values(("gas", "density"),
+    ...         ["temperature", "velocity_magnitude"])
+
+    """
+    def _func(self, arr):
+        return np.argmin(arr)
+
+class MinLocation(SampleAtMinFieldValues):
+    r"""
+    Calculates the minimum value plus the x, y, and z position of the minimum.
+
+    Parameters
+    ----------
+
+    field : tuple or string
         The field over which the extrema are to be calculated.
 
     Examples
@@ -588,32 +645,14 @@ class MinLocation(DerivedQuantity):
     >>> print ad.quantities.min_location(("gas", "density"))
 
     """
-    def count_values(self, *args, **kwargs):
-        self.num_vals = 5
-
     def __call__(self, field):
-        rv = super(MinLocation, self).__call__(field)
+        # Make sure we have an index
+        self.data_source.index
+        sample_fields = get_position_fields(field, self.data_source)
+        rv = super(MinLocation, self).__call__(field, sample_fields)
         if len(rv) == 1: rv = rv[0]
         return rv
 
-    def process_chunk(self, data, field):
-        field = data._determine_fields(field)[0]
-        ma = array_like_field(data, HUGE, field)
-        position_fields = get_position_fields(field, data)
-        mx = array_like_field(data, -1, position_fields[0])
-        my = array_like_field(data, -1, position_fields[1])
-        mz = array_like_field(data, -1, position_fields[2])
-        mini = -1
-        if data[field].size > 0:
-            mini = np.argmin(data[field])
-            ma = data[field][mini]
-            mx, my, mz = [data[ax][mini] for ax in position_fields]
-        return (ma, mini, mx, my, mz)
-
-    def reduce_intermediate(self, values):
-        i = np.argmin(values[0]) # ma is values[0]
-        return [val[i] for val in values]
-
 class SpinParameter(DerivedQuantity):
     r"""
     Calculates the dimensionless spin parameter.
@@ -621,7 +660,9 @@ class SpinParameter(DerivedQuantity):
     Given by Equation 3 of Peebles (1971, A&A, 11, 377), the spin parameter
     is defined as
 
-    lambda = (L \* |E|^(1/2)) / (G \* M^5/2),
+    .. math::
+
+      \lambda = (L * |E|^(1/2)) / (G * M^5/2),
 
     where L is the total angular momentum, E is the total energy (kinetic and
     potential), G is the gravitational constant, and M is the total mass.
@@ -658,7 +699,7 @@ class SpinParameter(DerivedQuantity):
         m = data.ds.quan(0., "g")
         if use_gas:
             e += (data["gas", "kinetic_energy"] *
-                  data["index", "cell_volume"]).sum(dtype=np.float64)
+                  data["gas", "cell_volume"]).sum(dtype=np.float64)
             j += data["gas", "angular_momentum_magnitude"].sum(dtype=np.float64)
             m += data["gas", "cell_mass"].sum(dtype=np.float64)
         if use_particles:
