@@ -14,30 +14,31 @@ Data containers based on geometric selection
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import types
 import numpy as np
-from contextlib import contextmanager
 
-from yt.funcs import *
-from yt.utilities.lib.alt_ray_tracers import cylindrical_ray_trace
-from yt.utilities.orientation import Orientation
-from .data_containers import \
+from yt.data_objects.data_containers import \
     YTSelectionContainer0D, YTSelectionContainer1D, \
     YTSelectionContainer2D, YTSelectionContainer3D
-from yt.data_objects.derived_quantities import \
-    DerivedQuantityCollection
+from yt.funcs import \
+    ensure_list, \
+    iterable, \
+    validate_width_tuple, \
+    fix_length
+from yt.geometry.selection_routines import \
+    points_in_cells
+from yt.units.yt_array import \
+    YTArray
 from yt.utilities.exceptions import \
     YTSphereTooSmall, \
     YTIllDefinedCutRegion, \
-    YTMixedCutRegion
-from yt.utilities.linear_interpolators import TrilinearFieldInterpolator
+    YTEllipsoidOrdering
 from yt.utilities.minimal_representation import \
     MinimalSliceData
 from yt.utilities.math_utils import get_rotation_matrix
-from yt.units.yt_array import YTQuantity
+from yt.utilities.orientation import Orientation
 
 
-class YTPointBase(YTSelectionContainer0D):
+class YTPoint(YTSelectionContainer0D):
     """
     A 0-dimensional object defined by a single point
 
@@ -51,8 +52,11 @@ class YTPointBase(YTSelectionContainer0D):
     ds: Dataset, optional
         An optional dataset to use rather than self.ds
     field_parameters : dictionary
-         A dictionary of field parameters than can be accessed by derived
-         fields.
+        A dictionary of field parameters than can be accessed by derived
+        fields.
+    data_source: optional
+        Draw the selection from the provided data source rather than
+        all data associated with the data_set
 
     Examples
     --------
@@ -64,11 +68,15 @@ class YTPointBase(YTSelectionContainer0D):
     """
     _type_name = "point"
     _con_args = ('p',)
-    def __init__(self, p, ds = None, field_parameters = None):
-        super(YTPointBase, self).__init__(ds, field_parameters)
-        self.p = p
+    def __init__(self, p, ds=None, field_parameters=None, data_source=None):
+        super(YTPoint, self).__init__(ds, field_parameters, data_source)
+        if isinstance(p, YTArray):
+            # we pass p through ds.arr to ensure code units are attached
+            self.p = self.ds.arr(p)
+        else:
+            self.p = self.ds.arr(p, 'code_length')
 
-class YTOrthoRayBase(YTSelectionContainer1D):
+class YTOrthoRay(YTSelectionContainer1D):
     """
     This is an orthogonal ray cast through the entire domain, at a specific
     coordinate.
@@ -85,13 +93,16 @@ class YTOrthoRayBase(YTSelectionContainer1D):
     coords : tuple of floats
         The (plane_x, plane_y) coordinates at which to cast the ray.  Note
         that this is in the plane coordinates: so if you are casting along
-        x, this will be (y,z).  If you are casting along y, this will be
-        (x,z).  If you are casting along z, this will be (x,y).
+        x, this will be (y, z).  If you are casting along y, this will be
+        (z, x).  If you are casting along z, this will be (x, y).
     ds: Dataset, optional
         An optional dataset to use rather than self.ds
     field_parameters : dictionary
          A dictionary of field parameters than can be accessed by derived
          fields.
+    data_source: optional
+        Draw the selection from the provided data source rather than
+        all data associated with the data_set
 
     Examples
     --------
@@ -100,12 +111,24 @@ class YTOrthoRayBase(YTSelectionContainer1D):
     >>> ds = yt.load("RedshiftOutput0005")
     >>> oray = ds.ortho_ray(0, (0.2, 0.74))
     >>> print oray["Density"]
+
+    Note: The low-level data representation for rays are not guaranteed to be 
+    spatially ordered.  In particular, with AMR datasets, higher resolution 
+    data is tagged on to the end of the ray.  If you want this data 
+    represented in a spatially ordered manner, manually sort it by the "t" 
+    field, which is the value of the parametric variable that goes from 0 at 
+    the start of the ray to 1 at the end:
+
+    >>> my_ray = ds.ortho_ray(...)
+    >>> ray_sort = np.argsort(my_ray["t"])
+    >>> density = my_ray["density"][ray_sort]
     """
     _key_fields = ['x','y','z','dx','dy','dz']
     _type_name = "ortho_ray"
     _con_args = ('axis', 'coords')
-    def __init__(self, axis, coords, ds=None, field_parameters=None):
-        super(YTOrthoRayBase, self).__init__(ds, field_parameters)
+    def __init__(self, axis, coords, ds=None, 
+                 field_parameters=None, data_source=None):
+        super(YTOrthoRay, self).__init__(ds, field_parameters, data_source)
         self.axis = axis
         xax = self.ds.coordinates.x_axis[self.axis]
         yax = self.ds.coordinates.y_axis[self.axis]
@@ -121,7 +144,7 @@ class YTOrthoRayBase(YTSelectionContainer1D):
     def coords(self):
         return (self.px, self.py)
 
-class YTRayBase(YTSelectionContainer1D):
+class YTRay(YTSelectionContainer1D):
     """
     This is an arbitrarily-aligned ray cast through the entire domain, at a
     specific coordinate.
@@ -144,6 +167,9 @@ class YTRayBase(YTSelectionContainer1D):
     field_parameters : dictionary
          A dictionary of field parameters than can be accessed by derived
          fields.
+    data_source: optional
+        Draw the selection from the provided data source rather than
+        all data associated with the data_set
 
     Examples
     --------
@@ -152,16 +178,39 @@ class YTRayBase(YTSelectionContainer1D):
     >>> ds = yt.load("RedshiftOutput0005")
     >>> ray = ds.ray((0.2, 0.74, 0.11), (0.4, 0.91, 0.31))
     >>> print ray["Density"], ray["t"], ray["dts"]
-    """
+
+    Note: The low-level data representation for rays are not guaranteed to be 
+    spatially ordered.  In particular, with AMR datasets, higher resolution 
+    data is tagged on to the end of the ray.  If you want this data 
+    represented in a spatially ordered manner, manually sort it by the "t" 
+    field, which is the value of the parametric variable that goes from 0 at 
+    the start of the ray to 1 at the end:
+
+    >>> my_ray = ds.ray(...)
+    >>> ray_sort = np.argsort(my_ray["t"])
+    >>> density = my_ray["density"][ray_sort]
+
+"""
     _type_name = "ray"
     _con_args = ('start_point', 'end_point')
     _container_fields = ("t", "dts")
-    def __init__(self, start_point, end_point, ds=None, field_parameters=None):
-        super(YTRayBase, self).__init__(ds, field_parameters)
-        self.start_point = self.ds.arr(start_point,
-                            'code_length', dtype='float64')
-        self.end_point = self.ds.arr(end_point,
-                            'code_length', dtype='float64')
+    def __init__(self, start_point, end_point, ds=None,
+                 field_parameters=None, data_source=None):
+        super(YTRay, self).__init__(ds, field_parameters, data_source)
+        if isinstance(start_point, YTArray):
+            self.start_point = \
+              self.ds.arr(start_point).to("code_length")
+        else:
+            self.start_point = \
+              self.ds.arr(start_point, 'code_length',
+                          dtype='float64')
+        if isinstance(end_point, YTArray):
+            self.end_point = \
+              self.ds.arr(end_point).to("code_length")
+        else:
+            self.end_point = \
+              self.ds.arr(end_point, 'code_length',
+                          dtype='float64')
         self.vec = self.end_point - self.start_point
         self._set_center(self.start_point)
         self.set_field_parameter('center', self.start_point)
@@ -177,7 +226,7 @@ class YTRayBase(YTSelectionContainer1D):
         else:
             raise KeyError(field)
 
-class YTSliceBase(YTSelectionContainer2D):
+class YTSlice(YTSelectionContainer2D):
     """
     This is a data object corresponding to a slice through the simulation
     domain.
@@ -191,7 +240,7 @@ class YTSliceBase(YTSelectionContainer2D):
 
     Parameters
     ----------
-    axis : int
+    axis : int or char
         The axis along which to slice.  Can be 0, 1, or 2 for x, y, z.
     coord : float
         The coordinate along the axis at which to slice.  This is in
@@ -204,6 +253,9 @@ class YTSliceBase(YTSelectionContainer2D):
     field_parameters : dictionary
          A dictionary of field parameters than can be accessed by derived
          fields.
+    data_source: optional
+        Draw the selection from the provided data source rather than
+        all data associated with the data_set
 
     Examples
     --------
@@ -217,10 +269,10 @@ class YTSliceBase(YTSelectionContainer2D):
     _type_name = "slice"
     _con_args = ('axis', 'coord')
     _container_fields = ("px", "py", "pdx", "pdy")
-
     def __init__(self, axis, coord, center=None, ds=None,
-                 field_parameters = None):
-        YTSelectionContainer2D.__init__(self, axis, ds, field_parameters)
+                 field_parameters=None, data_source=None):
+        YTSelectionContainer2D.__init__(self, axis, ds,
+                                        field_parameters, data_source)
         self._set_center(center)
         self.coord = coord
 
@@ -258,7 +310,26 @@ class YTSliceBase(YTSelectionContainer2D):
         pw = self._get_pw(fields, center, width, origin, 'Slice')
         return pw
 
-class YTCuttingPlaneBase(YTSelectionContainer2D):
+    def plot(self, fields=None):
+        if hasattr(self._data_source, "left_edge") and \
+            hasattr(self._data_source, "right_edge"):
+            left_edge = self._data_source.left_edge
+            right_edge = self._data_source.right_edge
+            center = (left_edge + right_edge)/2.0
+            width = right_edge - left_edge
+            xax = self.ds.coordinates.x_axis[self.axis]
+            yax = self.ds.coordinates.y_axis[self.axis]
+            lx, rx = left_edge[xax], right_edge[xax]
+            ly, ry = left_edge[yax], right_edge[yax]
+            width = (rx-lx), (ry-ly)
+        else:
+            width = self.ds.domain_width
+            center = self.ds.domain_center
+        pw = self._get_pw(fields, center, width, 'native', 'Slice')
+        pw.show()
+        return pw
+
+class YTCuttingPlane(YTSelectionContainer2D):
     """
     This is a data object corresponding to an oblique slice through the
     simulation domain.
@@ -285,13 +356,16 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
     field_parameters : dictionary
          A dictionary of field parameters than can be accessed by derived
          fields.
+    data_source: optional
+        Draw the selection from the provided data source rather than
+        all data associated with the data_set
 
     Notes
     -----
 
     This data object in particular can be somewhat expensive to create.
     It's also important to note that unlike the other 2D data objects, this
-    oject provides px, py, pz, as some cells may have a height from the
+    object provides px, py, pz, as some cells may have a height from the
     plane.
 
     Examples
@@ -307,11 +381,13 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
     _key_fields = YTSelectionContainer2D._key_fields + ['pz','pdz']
     _type_name = "cutting"
     _con_args = ('normal', 'center')
+    _tds_attrs = ("_inv_mat",)
+    _tds_fields = ("x", "y", "z", "dx")
     _container_fields = ("px", "py", "pz", "pdx", "pdy", "pdz")
-
-    def __init__(self, normal, center, north_vector = None, 
-                 ds = None, field_parameters = None):
-        YTSelectionContainer2D.__init__(self, 4, ds, field_parameters)
+    def __init__(self, normal, center, north_vector=None,
+                 ds=None, field_parameters=None, data_source=None):
+        YTSelectionContainer2D.__init__(self, 4, ds,
+                                        field_parameters, data_source)
         self._set_center(center)
         self.set_field_parameter('center',center)
         # Let's set up our plane equation
@@ -322,7 +398,6 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
         self._x_vec = self.orienter.unit_vectors[0]
         self._y_vec = self.orienter.unit_vectors[1]
         # First we try all three, see which has the best result:
-        vecs = np.identity(3)
         self._rot_mat = np.array([self._x_vec,self._y_vec,self._norm_vec])
         self._inv_mat = np.linalg.pinv(self._rot_mat)
         self.set_field_parameter('cp_x_vec',self._x_vec)
@@ -333,61 +408,6 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
     def normal(self):
         return self._norm_vec
 
-    def to_frb(self, width, resolution, height=None,
-               periodic=False):
-        r"""This function returns an ObliqueFixedResolutionBuffer generated
-        from this object.
-
-        An ObliqueFixedResolutionBuffer is an object that accepts a
-        variable-resolution 2D object and transforms it into an NxM bitmap that
-        can be plotted, examined or processed.  This is a convenience function
-        to return an FRB directly from an existing 2D data object.  Unlike the
-        corresponding to_frb function for other YTSelectionContainer2D objects, 
-        this does not accept a 'center' parameter as it is assumed to be 
-        centered at the center of the cutting plane.
-
-        Parameters
-        ----------
-        width : width specifier
-            This can either be a floating point value, in the native domain
-            units of the simulation, or a tuple of the (value, unit) style.
-            This will be the width of the FRB.
-        height : height specifier, optional
-            This will be the height of the FRB, by default it is equal to width.
-        resolution : int or tuple of ints
-            The number of pixels on a side of the final FRB.
-
-        Returns
-        -------
-        frb : :class:`~yt.visualization.fixed_resolution.ObliqueFixedResolutionBuffer`
-            A fixed resolution buffer, which can be queried for fields.
-
-        Examples
-        --------
-
-        >>> v, c = ds.find_max("density")
-        >>> sp = ds.sphere(c, (100.0, 'au'))
-        >>> L = sp.quantities.angular_momentum_vector()
-        >>> cutting = ds.cutting(L, c)
-        >>> frb = cutting.to_frb( (1.0, 'pc'), 1024)
-        >>> write_image(np.log10(frb["Density"]), 'density_1pc.png')
-        """
-        if iterable(width):
-            w, u = width
-            width = self.ds.quan(w, input_units = u)
-        if height is None:
-            height = width
-        elif iterable(height):
-            h, u = height
-            height = self.ds.quan(w, input_units = u)
-        if not iterable(resolution):
-            resolution = (resolution, resolution)
-        from yt.visualization.fixed_resolution import ObliqueFixedResolutionBuffer
-        bounds = (-width/2.0, width/2.0, -height/2.0, height/2.0)
-        frb = ObliqueFixedResolutionBuffer(self, bounds, resolution,
-                    periodic = periodic)
-        return frb
-
     def _generate_container_field(self, field):
         if self._current_chunk is None:
             self.index._identify_base_chunk(self)
@@ -455,7 +475,7 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
         pw._setup_plots()
         return pw
 
-    def to_frb(self, width, resolution, height=None):
+    def to_frb(self, width, resolution, height=None, periodic=False):
         r"""This function returns an ObliqueFixedResolutionBuffer generated
         from this object.
 
@@ -477,6 +497,9 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
             This will be the height of the FRB, by default it is equal to width.
         resolution : int or tuple of ints
             The number of pixels on a side of the final FRB.
+        periodic : boolean
+            This can be true or false, and governs whether the pixelization
+            will span the domain boundaries.
 
         Returns
         -------
@@ -505,10 +528,11 @@ class YTCuttingPlaneBase(YTSelectionContainer2D):
             resolution = (resolution, resolution)
         from yt.visualization.fixed_resolution import ObliqueFixedResolutionBuffer
         bounds = (-width/2.0, width/2.0, -height/2.0, height/2.0)
-        frb = ObliqueFixedResolutionBuffer(self, bounds, resolution)
+        frb = ObliqueFixedResolutionBuffer(self, bounds, resolution,
+                                           periodic=periodic)
         return frb
 
-class YTDiskBase(YTSelectionContainer3D):
+class YTDisk(YTSelectionContainer3D):
     """
     By providing a *center*, a *normal*, a *radius* and a *height* we
     can define a cylinder of any proportion.  Only cells whose centers are
@@ -516,7 +540,7 @@ class YTDiskBase(YTSelectionContainer3D):
 
     Parameters
     ----------
-    center : array_like 
+    center : array_like
         coordinate to which the normal, radius, and height all reference
     normal : array_like
         the normal vector defining the direction of lengthwise part of the 
@@ -533,6 +557,9 @@ class YTDiskBase(YTSelectionContainer3D):
     field_parameters : dictionary
          A dictionary of field parameters than can be accessed by derived
          fields.
+    data_source: optional
+        Draw the selection from the provided data source rather than
+        all data associated with the data_set
 
     Examples
     --------
@@ -545,8 +572,9 @@ class YTDiskBase(YTSelectionContainer3D):
     _type_name = "disk"
     _con_args = ('center', '_norm_vec', 'radius', 'height')
     def __init__(self, center, normal, radius, height, fields=None,
-                 ds=None, **kwargs):
-        YTSelectionContainer3D.__init__(self, center, fields, ds, **kwargs)
+                 ds=None, field_parameters=None, data_source=None):
+        YTSelectionContainer3D.__init__(self, center, ds,
+                                        field_parameters, data_source)
         self._norm_vec = np.array(normal)/np.sqrt(np.dot(normal,normal))
         self.set_field_parameter("normal", self._norm_vec)
         self.set_field_parameter("center", self.center)
@@ -554,7 +582,7 @@ class YTDiskBase(YTSelectionContainer3D):
         self.radius = fix_length(radius, self.ds)
         self._d = -1.0 * np.dot(self._norm_vec, self.center)
 
-class YTRegionBase(YTSelectionContainer3D):
+class YTRegion(YTSelectionContainer3D):
     """A 3D region of data with an arbitrary center.
 
     Takes an array of three *left_edge* coordinates, three
@@ -574,9 +602,10 @@ class YTRegionBase(YTSelectionContainer3D):
     """
     _type_name = "region"
     _con_args = ('center', 'left_edge', 'right_edge')
-    def __init__(self, center, left_edge, right_edge, fields = None,
-                 ds = None, **kwargs):
-        YTSelectionContainer3D.__init__(self, center, ds, **kwargs)
+    def __init__(self, center, left_edge, right_edge, fields=None,
+                 ds=None, field_parameters=None, data_source=None):
+        YTSelectionContainer3D.__init__(self, center, ds,
+                                        field_parameters, data_source)
         if not isinstance(left_edge, YTArray):
             self.left_edge = self.ds.arr(left_edge, 'code_length')
         else:
@@ -586,20 +615,22 @@ class YTRegionBase(YTSelectionContainer3D):
         else:
             self.right_edge = right_edge
 
-class YTDataCollectionBase(YTSelectionContainer3D):
+class YTDataCollection(YTSelectionContainer3D):
     """
     By selecting an arbitrary *object_list*, we can act on those grids.
     Child cells are not returned.
     """
     _type_name = "data_collection"
     _con_args = ("_obj_list",)
-    def __init__(self, center, obj_list, ds = None, field_parameters = None):
-        YTSelectionContainer3D.__init__(self, center, ds, field_parameters)
+    def __init__(self, obj_list, ds=None, field_parameters=None,
+                 data_source=None, center=None):
+        YTSelectionContainer3D.__init__(self, center, ds,
+                                        field_parameters, data_source)
         self._obj_ids = np.array([o.id - o._id_offset for o in obj_list],
                                 dtype="int64")
         self._obj_list = obj_list
 
-class YTSphereBase(YTSelectionContainer3D):
+class YTSphere(YTSelectionContainer3D):
     """
     A sphere of points defined by a *center* and a *radius*.
 
@@ -620,8 +651,10 @@ class YTSphereBase(YTSelectionContainer3D):
     """
     _type_name = "sphere"
     _con_args = ('center', 'radius')
-    def __init__(self, center, radius, ds = None, field_parameters = None):
-        super(YTSphereBase, self).__init__(center, ds, field_parameters)
+    def __init__(self, center, radius, ds=None,
+                 field_parameters=None, data_source=None):
+        super(YTSphere, self).__init__(center, ds,
+                                           field_parameters, data_source)
         # Unpack the radius, if necessary
         radius = fix_length(radius, self.ds)
         if radius < self.index.get_smallest_dx():
@@ -631,7 +664,7 @@ class YTSphereBase(YTSelectionContainer3D):
         self.set_field_parameter("center", self.center)
         self.radius = radius
 
-class YTEllipsoidBase(YTSelectionContainer3D):
+class YTEllipsoid(YTSelectionContainer3D):
     """
     By providing a *center*,*A*,*B*,*C*,*e0*,*tilt* we
     can define a ellipsoid of any proportion.  Only cells whose
@@ -666,8 +699,9 @@ class YTEllipsoidBase(YTSelectionContainer3D):
     _type_name = "ellipsoid"
     _con_args = ('center', '_A', '_B', '_C', '_e0', '_tilt')
     def __init__(self, center, A, B, C, e0, tilt, fields=None,
-                 ds=None, field_parameters = None):
-        YTSelectionContainer3D.__init__(self, center, ds, field_parameters)
+                 ds=None, field_parameters=None, data_source=None):
+        YTSelectionContainer3D.__init__(self, center, ds,
+                                        field_parameters, data_source)
         # make sure the magnitudes of semi-major axes are in order
         if A<B or B<C:
             raise YTEllipsoidOrdering(ds, A, B, C)
@@ -676,10 +710,10 @@ class YTEllipsoidBase(YTSelectionContainer3D):
         self._B = self.ds.quan(B, 'code_length')
         self._C = self.ds.quan(C, 'code_length')
         if self._C < self.index.get_smallest_dx():
-            raise YTSphereTooSmall(ds, self._C, self.index.get_smallest_dx())
+            raise YTSphereTooSmall(self.ds, self._C, self.index.get_smallest_dx())
         self._e0 = e0 = e0 / (e0**2.0).sum()**0.5
         self._tilt = tilt
-        
+ 
         # find the t1 angle needed to rotate about z axis to align e0 to x
         t1 = np.arctan(e0[1] / e0[0])
         # rotate e0 by -t1
@@ -710,14 +744,14 @@ class YTEllipsoidBase(YTSelectionContainer3D):
         self.set_field_parameter('e1', e1)
         self.set_field_parameter('e2', e2)
 
-class YTCutRegionBase(YTSelectionContainer3D):
+class YTCutRegion(YTSelectionContainer3D):
     """
     This is a data object designed to allow individuals to apply logical
     operations to fields and filter as a result of those cuts.
 
     Parameters
     ----------
-    base_object : YTSelectionContainer3D
+    data_source : YTSelectionContainer3D
         The object to which cuts will be applied.
     conditionals : list of strings
         A list of conditionals that will be evaluated.  In the namespace
@@ -730,30 +764,34 @@ class YTCutRegionBase(YTSelectionContainer3D):
 
     >>> import yt
     >>> ds = yt.load("RedshiftOutput0005")
-    >>> sp = ds.sphere("max", (1.0, 'mpc'))
+    >>> sp = ds.sphere("max", (1.0, 'Mpc'))
     >>> cr = ds.cut_region(sp, ["obj['temperature'] < 1e3"])
     """
     _type_name = "cut_region"
     _con_args = ("base_object", "conditionals")
-    def __init__(self, base_object, conditionals, ds = None,
-                 field_parameters = None):
-        super(YTCutRegionBase, self).__init__(base_object.center, ds, field_parameters)
+    def __init__(self, data_source, conditionals, ds=None,
+                 field_parameters=None, base_object=None):
+        if base_object is not None:
+            # passing base_object explicitly has been deprecated,
+            # but we handle it here for backward compatibility
+            if data_source is not None:
+                raise RuntimeError(
+                    "Cannot use both base_object and data_source")
+            data_source=base_object
+        super(YTCutRegion, self).__init__(
+            data_source.center, ds, field_parameters, data_source=data_source)
         self.conditionals = ensure_list(conditionals)
-        self.base_object = base_object
+        self.base_object = data_source
         self._selector = None
         # Need to interpose for __getitem__, fwidth, fcoords, icoords, iwidth,
         # ires and get_data
 
-    @property
-    def selector(self):
-        raise NotImplementedError
-
     def chunks(self, fields, chunking_style, **kwargs):
         # We actually want to chunk the sub-chunk, not ourselves.  We have no
         # chunks to speak of, as we do not data IO.
         for chunk in self.index._chunk(self.base_object,
-                                           chunking_style,
-                                           **kwargs):
+                                       chunking_style,
+                                       **kwargs):
             with self.base_object._chunked_read(chunk):
                 with self._chunked_read(chunk):
                     self.get_data(fields)
@@ -766,8 +804,10 @@ class YTCutRegionBase(YTSelectionContainer3D):
         for field in fields:
             f = self.base_object[field]
             if f.shape != ind.shape:
-                raise YTMixedCutRegion(self.conditionals, field)
-            self.field_data[field] = self.base_object[field][ind]
+                parent = getattr(self, "parent", self.base_object)
+                self.field_data[field] = parent[field][self._part_ind]
+            else:
+                self.field_data[field] = self.base_object[field][ind]
 
     @property
     def blocks(self):
@@ -782,9 +822,6 @@ class YTCutRegionBase(YTSelectionContainer3D):
             if not np.any(m): continue
             yield obj, m
 
-    def cut_region(self, *args, **kwargs):
-        raise NotImplementedError
-
     @property
     def _cond_ind(self):
         ind = None
@@ -798,6 +835,22 @@ class YTCutRegionBase(YTSelectionContainer3D):
                 np.logical_and(res, ind, ind)
         return ind
 
+    _particle_mask = None
+    @property
+    def _part_ind(self):
+        if self._particle_mask is None:
+            parent = getattr(self, "parent", self.base_object)
+            units = "code_length"
+            mask = points_in_cells(
+                self["x"].to(units), self["y"].to(units),
+                self["z"].to(units), self["dx"].to(units),
+                self["dy"].to(units), self["dz"].to(units),
+                parent["particle_position_x"].to(units),
+                parent["particle_position_y"].to(units),
+                parent["particle_position_z"].to(units))
+            self._particle_mask = mask
+        return self._particle_mask
+
     @property
     def icoords(self):
         return self.base_object.icoords[self._cond_ind,:]
@@ -813,4 +866,3 @@ class YTCutRegionBase(YTSelectionContainer3D):
     @property
     def fwidth(self):
         return self.base_object.fwidth[self._cond_ind,:]
-
