@@ -15,20 +15,18 @@ Here are some fields that are specific to fluids.
 
 import numpy as np
 
-from yt.funcs import \
-    just_one
+from yt.units.unit_object import Unit
 
 from .derived_field import \
-    ValidateParameter, \
     ValidateSpatial
 
 from .field_plugin_registry import \
     register_field_plugin
 
 from .vector_operations import \
-     create_averaged_field, \
-     create_magnitude_field, \
-     create_vector_fields
+    create_averaged_field, \
+    create_magnitude_field, \
+    create_vector_fields
 
 from yt.utilities.physical_constants import \
     mh, \
@@ -37,20 +35,6 @@ from yt.utilities.physical_constants import \
 from yt.utilities.physical_ratios import \
     metallicity_sun
 
-from yt.units.yt_array import \
-    YTArray
-
-from yt.utilities.math_utils import \
-    get_sph_r_component, \
-    get_sph_theta_component, \
-    get_sph_phi_component, \
-    get_cyl_r_component, \
-    get_cyl_z_component, \
-    get_cyl_theta_component, \
-    get_cyl_r, get_cyl_theta, \
-    get_cyl_z, get_sph_r, \
-    get_sph_theta, get_sph_phi, \
-    periodic_dist, euclidean_dist
 
 @register_field_plugin
 def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
@@ -67,21 +51,24 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
     else:
         sl_left, sl_right, div_fac = slice_info
 
-    create_vector_fields(registry, "velocity", "cm / s", ftype, slice_info)
+    unit_system = registry.ds.unit_system
+
+    create_vector_fields(registry, "velocity", unit_system["velocity"], ftype, slice_info)
+    create_vector_fields(registry, "magnetic_field", unit_system["magnetic_field"], ftype, slice_info)
 
     def _cell_mass(field, data):
-        return data[ftype, "density"] * data["index", "cell_volume"]
+        return data[ftype, "density"] * data[ftype, "cell_volume"]
 
     registry.add_field((ftype, "cell_mass"),
         function=_cell_mass,
-        units="g")
+        units=unit_system["mass"])
 
     def _sound_speed(field, data):
         tr = data.ds.gamma * data[ftype, "pressure"] / data[ftype, "density"]
         return np.sqrt(tr)
     registry.add_field((ftype, "sound_speed"),
              function=_sound_speed,
-             units="cm/s")
+             units=unit_system["velocity"])
 
     def _radial_mach_number(field, data):
         """ Radial component of M{|v|/c_sound} """
@@ -95,9 +82,9 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         return 0.5*data[ftype, "density"] * ( data[ftype, "velocity_x"]**2.0
                                               + data[ftype, "velocity_y"]**2.0
                                               + data[ftype, "velocity_z"]**2.0 )
-    registry.add_field(("gas", "kinetic_energy"),
+    registry.add_field((ftype, "kinetic_energy"),
                        function = _kin_energy,
-                       units = "erg / cm**3")
+                       units = unit_system["pressure"])
 
     def _mach_number(field, data):
         """ M{|v|/c_sound} """
@@ -107,18 +94,18 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
             units = "")
 
     def _courant_time_step(field, data):
-        t1 = data["index", "dx"] / (data[ftype, "sound_speed"]
+        t1 = data[ftype, "dx"] / (data[ftype, "sound_speed"]
                         + np.abs(data[ftype, "velocity_x"]))
-        t2 = data["index", "dy"] / (data[ftype, "sound_speed"]
+        t2 = data[ftype, "dy"] / (data[ftype, "sound_speed"]
                         + np.abs(data[ftype, "velocity_y"]))
-        t3 = data["index", "dz"] / (data[ftype, "sound_speed"]
+        t3 = data[ftype, "dz"] / (data[ftype, "sound_speed"]
                         + np.abs(data[ftype, "velocity_z"]))
         tr = np.minimum(np.minimum(t1, t2), t3)
         return tr
 
     registry.add_field((ftype, "courant_time_step"),
              function=_courant_time_step,
-             units="s")
+             units=unit_system["time"])
 
     def _pressure(field, data):
         """ M{(Gamma-1.0)*rho*E} """
@@ -128,7 +115,7 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
 
     registry.add_field((ftype, "pressure"),
              function=_pressure,
-             units="dyne/cm**2")
+             units=unit_system["pressure"])
 
     def _kT(field, data):
         return (kboltz*data[ftype, "temperature"]).in_units("keV")
@@ -158,10 +145,10 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
              units="Zsun")
 
     def _metal_mass(field, data):
-        return data[ftype, "metal_density"] * data["index", "cell_volume"]
+        return data[ftype, "metal_density"] * data[ftype, "cell_volume"]
     registry.add_field((ftype, "metal_mass"),
                        function=_metal_mass,
-                       units="g")
+                       units=unit_system["mass"])
 
     def _number_density(field, data):
         field_data = np.zeros_like(data["gas", "%s_number_density" % \
@@ -171,7 +158,7 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
         return field_data
     registry.add_field((ftype, "number_density"),
                        function = _number_density,
-                       units="cm**-3")
+                       units=unit_system["number_density"])
     
     def _mean_molecular_weight(field, data):
         return (data[ftype, "density"] / (mh * data[ftype, "number_density"]))
@@ -179,13 +166,13 @@ def setup_fluid_fields(registry, ftype = "gas", slice_info = None):
               function=_mean_molecular_weight,
               units="")
 
-    setup_gradient_fields(registry, (ftype, "pressure"), "dyne/cm**2",
+    setup_gradient_fields(registry, (ftype, "pressure"), unit_system["pressure"],
                           slice_info)
 
-    setup_gradient_fields(registry, (ftype, "density"), "g / cm**3",
+    setup_gradient_fields(registry, (ftype, "density"), unit_system["density"],
                           slice_info)
 
-    create_averaged_field(registry, "density", "g/cm**3",
+    create_averaged_field(registry, "density", unit_system["density"],
                           ftype=ftype, slice_info=slice_info,
                           weight="cell_mass")
 
@@ -206,7 +193,7 @@ def setup_gradient_fields(registry, grad_field, field_units, slice_info = None):
         slice_3dl[axi] = sl_left
         slice_3dr[axi] = sl_right
         def func(field, data):
-            ds = div_fac * data["index", "d%s" % ax]
+            ds = div_fac * data[ftype, "d%s" % ax]
             f  = data[grad_field][slice_3dr]/ds[slice_3d]
             f -= data[grad_field][slice_3dl]/ds[slice_3d]
             new_field = data.ds.arr(np.zeros_like(data[grad_field], dtype=np.float64),
@@ -215,10 +202,8 @@ def setup_gradient_fields(registry, grad_field, field_units, slice_info = None):
             return new_field
         return func
 
-    if field_units != "":
-        grad_units = "(%s) / cm" % field_units
-    else:
-        grad_units = "1 / cm"
+    field_units = Unit(field_units, registry=registry.ds.unit_registry)
+    grad_units = field_units / registry.ds.unit_system["length"]
 
     for axi, ax in enumerate('xyz'):
         f = grad_func(axi, ax)
