@@ -48,9 +48,9 @@ class Orion2FieldInfo(ChomboFieldInfo):
         ("Y-momentum", (mom_units, ["momentum_y"], None)),
         ("Z-momentum", (mom_units, ["momentum_z"], None)),
         ("temperature", ("K", ["temperature"], None)),
-        ("X-magnfield", (b_units, ["magnetic_field_x"], None)),
-        ("Y-magnfield", (b_units, ["magnetic_field_y"], None)),
-        ("Z-magnfield", (b_units, ["magnetic_field_z"], None)),
+        ("X-magnfield", (b_units, [], None)),
+        ("Y-magnfield", (b_units, [], None)),
+        ("Z-magnfield", (b_units, [], None)),
         ("directrad-dedt-density", (eden_units, ["directrad-dedt-density"], None)),
         ("directrad-dpxdt-density", (mom_units, ["directrad-dpxdt-density"], None)),
         ("directrad-dpydt-density", (mom_units, ["directrad-dpydt-density"], None)),
@@ -78,29 +78,47 @@ class Orion2FieldInfo(ChomboFieldInfo):
         ("particle_id", ("", ["particle_index"], None)),
     )
 
+    def setup_particle_fields(self, ptype):
+
+        def _get_vel(axis):
+            def velocity(field, data):
+                return data["particle_momentum_%s" % axis]/data["particle_mass"]
+            return velocity
+
+        for ax in 'xyz':
+            self.add_field((ptype, "particle_velocity_%s" % ax), 
+                           function=_get_vel(ax),
+                           particle_type=True,
+                           units="code_length/code_time")
+
+        super(Orion2FieldInfo, self).setup_particle_fields(ptype)
+
     def setup_fluid_fields(self):
+        from yt.fields.magnetic_field import \
+            setup_magnetic_field_aliases
+        unit_system = self.ds.unit_system
         def _thermal_energy_density(field, data):
             try:
-                return data['energy-density'] - data['kinetic_energy_density'] - \
-                    data["magnetic_energy_density"]
+                return data["energy-density"] - data["kinetic_energy"] - \
+                    data["magnetic_energy"]
             except YTFieldNotFound:
-                return data['energy-density'] - data['kinetic_energy_density']
+                return data['energy-density'] - data["kinetic_energy"]
 
         def _thermal_energy(field, data):
             return data['thermal_energy_density']/data['density']
 
-        def _magnetic_energy_density(field, data):
+        def _magnetic_energy(field, data):
             ret = data["X-magnfield"]**2
             if data.ds.dimensionality > 1:
                 ret = ret + data["Y-magnfield"]**2
             if data.ds.dimensionality > 2:
                 ret = ret + data["Z-magnfield"]**2
-            return ret/2.0
+            return ret/8.0/np.pi
 
-        def _magnetic_energy(field, data):
-            return data['magnetic_energy_density']/data['density']
+        def _specific_magnetic_energy(field, data):
+            return data['specific_magnetic_energy']/data['density']
 
-        def _kinetic_energy_density(field, data):
+        def _kinetic_energy(field, data):
             p2 = data['X-momentum']**2
             if data.ds.dimensionality > 1:
                 p2 = p2 + data["Y-momentum"]**2
@@ -108,8 +126,8 @@ class Orion2FieldInfo(ChomboFieldInfo):
                 p2 = p2 + data["Z-momentum"]**2
             return 0.5 * p2/data['density']
 
-        def _kinetic_energy(field, data):
-            return data['kinetic_energy_density']/data['density']
+        def _specific_kinetic_energy(field, data):
+            return data['kinetic_energy']/data['density']
 
         def _temperature(field, data):
             c_v = data.ds.quan(data.ds.parameters['radiation.const_cv'], 
@@ -123,27 +141,29 @@ class Orion2FieldInfo(ChomboFieldInfo):
 
         for ax in 'xyz':
             self.add_field(("gas", "velocity_%s" % ax), function = _get_vel(ax),
-                           units = "cm/s")
+                           units = unit_system["velocity"])
         self.add_field(("gas", "thermal_energy"),
                        function = _thermal_energy,
-                       units = "erg/g")
+                       units = unit_system["specific_energy"])
         self.add_field(("gas", "thermal_energy_density"),
                        function = _thermal_energy_density,
-                       units = "erg/cm**3")
+                       units = unit_system["pressure"])
         self.add_field(("gas", "kinetic_energy"),
                        function = _kinetic_energy,
-                       units = "erg/g")
-        self.add_field(("gas", "kinetic_energy_density"),
-                       function = _kinetic_energy_density,
-                       units = "erg/cm**3")
+                       units = unit_system["pressure"])
+        self.add_field(("gas", "specific_kinetic_energy"),
+                       function = _specific_kinetic_energy,
+                       units = unit_system["specific_energy"])
         self.add_field(("gas", "magnetic_energy"),
                        function = _magnetic_energy,
-                       units = "erg/g")
-        self.add_field(("gas", "magnetic_energy_density"),
-                       function = _magnetic_energy_density,
-                       units = "erg/cm**3")
+                       units = unit_system["pressure"])
+        self.add_field(("gas", "specific_magnetic_energy"),
+                       function = _specific_magnetic_energy,
+                       units = unit_system["specific_energy"])
         self.add_field(("gas", "temperature"), function=_temperature,
-                       units="K")
+                       units=unit_system["temperature"])
+
+        setup_magnetic_field_aliases(self, "chombo", ["%s-magnfield" % ax for ax in "XYZ"])
 
 
 class ChomboPICFieldInfo3D(FieldInfoContainer):
@@ -298,9 +318,15 @@ class PlutoFieldInfo(ChomboFieldInfo):
         ("vx1", (vel_units, ["velocity_x"], None)),
         ("vx2", (vel_units, ["velocity_y"], None)),
         ("vx3", (vel_units, ["velocity_z"], None)),
-        ("bx1", (b_units, ["magnetic_field_x"], None)),
-        ("bx2", (b_units, ["magnetic_field_y"], None)),
-        ("bx3", (b_units, ["magnetic_field_z"], None)),
+        ("bx1", (b_units, [], None)),
+        ("bx2", (b_units, [], None)),
+        ("bx3", (b_units, [], None)),
     )
 
     known_particle_fields = ()
+
+    def setup_fluid_fields(self):
+        from yt.fields.magnetic_field import \
+            setup_magnetic_field_aliases
+        setup_magnetic_field_aliases(self, "chombo", ["bx%s" % ax for ax in [1,2,3]])
+
