@@ -13,19 +13,23 @@ Data structures for Enzo
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import h5py
+from yt.utilities.on_demand_imports import _h5py as h5py
 import weakref
 import numpy as np
 import os
 import stat
 import string
+import time
 import re
 
-from threading import Thread
-
+from collections import defaultdict
 from yt.extern.six.moves import zip as izip
 
-from yt.funcs import *
+from yt.funcs import \
+    ensure_list, \
+    ensure_tuple, \
+    get_pbar, \
+    setdefaultattr
 from yt.config import ytcfg
 from yt.data_objects.grid_patch import \
     AMRGridPatch
@@ -36,20 +40,15 @@ from yt.geometry.geometry_handler import \
 from yt.data_objects.static_output import \
     Dataset
 from yt.fields.field_info_container import \
-    FieldInfoContainer, NullFunc
-from yt.utilities.definitions import \
-    mpc_conversion, sec_conversion
-from yt.utilities.physical_constants import \
+    NullFunc
+from yt.utilities.physical_ratios import \
     rho_crit_g_cm3_h2, cm_per_mpc
-from yt.utilities.io_handler import io_registry
 from yt.utilities.logger import ytLogger as mylog
 from yt.utilities.pyparselibconfig import libconfig
 
 from .fields import \
     EnzoFieldInfo
 
-from yt.utilities.parallel_tools.parallel_analysis_interface import \
-    parallel_blocking_call
 
 class EnzoGrid(AMRGridPatch):
     """
@@ -77,7 +76,6 @@ class EnzoGrid(AMRGridPatch):
         """
         rf = self.ds.refine_by
         my_ind = self.id - self._id_offset
-        le = self.LeftEdge
         self.dds = self.Parent.dds/rf
         ParentLeftIndex = np.rint((self.LeftEdge-self.Parent.LeftEdge)/self.Parent.dds)
         self.start_index = rf*(ParentLeftIndex + self.Parent.get_global_startindex()).astype('int64')
@@ -148,12 +146,9 @@ class EnzoGridGZ(EnzoGrid):
         # We will attempt this by creating a datacube that is exactly bigger
         # than the grid by nZones*dx in each direction
         nl = self.get_global_startindex() - n_zones
-        nr = nl + self.ActiveDimensions + 2*n_zones
         new_left_edge = nl * self.dds + self.ds.domain_left_edge
-        new_right_edge = nr * self.dds + self.ds.domain_left_edge
         # Something different needs to be done for the root grid, though
         level = self.Level
-        args = (level, new_left_edge, new_right_edge)
         kwargs = {'dims': self.ActiveDimensions + 2*n_zones,
                   'num_ghost_zones':n_zones,
                   'use_pbar':False}
@@ -197,7 +192,7 @@ class EnzoHierarchy(GridIndex):
     def __init__(self, ds, dataset_type):
 
         self.dataset_type = dataset_type
-        if ds.file_style != None:
+        if ds.file_style is not None:
             self._bn = ds.file_style
         else:
             self._bn = "%s.cpu%%04i"
@@ -268,14 +263,12 @@ class EnzoHierarchy(GridIndex):
             for line in f:
                 if line.startswith(token):
                     return line.split()[2:]
-        t1 = time.time()
         pattern = r"Pointer: Grid\[(\d*)\]->NextGrid(Next|This)Level = (\d*)\s+$"
         patt = re.compile(pattern)
         f = open(self.index_filename, "rt")
         self.grids = [self.grid(1, self)]
         self.grids[0].Level = 0
         si, ei, LE, RE, fn, npart = [], [], [], [], [], []
-        all = [si, ei, LE, RE, fn]
         pbar = get_pbar("Parsing Hierarchy ", self.num_grids)
         version = self.dataset.parameters.get("VersionNumber", None)
         params = self.dataset.parameters
@@ -285,14 +278,15 @@ class EnzoHierarchy(GridIndex):
             active_particles = True
             nap = dict((ap_type, []) for ap_type in 
                 params["Physics"]["ActiveParticles"]["ActiveParticlesEnabled"])
-        elif version == 2.2:
-            active_particles = True
-            nap = {}
-            for type in self.parameters.get("AppendActiveParticleType", []):
-                nap[type] = []
         else:
-            active_particles = False
-            nap = None
+            if "AppendActiveParticleType" in self.parameters:
+                nap = {}
+                active_particles = True
+                for type in self.parameters.get("AppendActiveParticleType", []):
+                    nap[type] = []
+            else:
+                nap = None
+                active_particles = False
         for grid_id in range(self.num_grids):
             pbar.update(grid_id)
             # We will unroll this list
@@ -326,7 +320,6 @@ class EnzoHierarchy(GridIndex):
         temp_grids[:] = self.grids
         self.grids = temp_grids
         self.filenames = fn
-        t2 = time.time()
 
     def _initialize_grid_arrays(self):
         super(EnzoHierarchy, self)._initialize_grid_arrays()
@@ -403,7 +396,7 @@ class EnzoHierarchy(GridIndex):
         fields = []
         for ptype in self.dataset["AppendActiveParticleType"]:
             select_grids = self.grid_active_particle_count[ptype].flat
-            if np.any(select_grids) == False:
+            if not np.any(select_grids):
                 current_ptypes = self.dataset.particle_types
                 new_ptypes = [p for p in current_ptypes if p != ptype]
                 self.dataset.particle_types = new_ptypes
@@ -452,6 +445,16 @@ class EnzoHierarchy(GridIndex):
             if "AppendActiveParticleType" in self.dataset.parameters:
                 ap_fields = self._detect_active_particle_fields()
                 field_list = list(set(field_list).union(ap_fields))
+                if not any(f[0] == 'io' for f in field_list):
+                    if 'io' in self.dataset.particle_types_raw:
+                        ptypes_raw = list(self.dataset.particle_types_raw)
+                        ptypes_raw.remove('io')
+                        self.dataset.particle_types_raw = tuple(ptypes_raw)
+
+                    if 'io' in self.dataset.particle_types:
+                        ptypes = list(self.dataset.particle_types)
+                        ptypes.remove('io')
+                        self.dataset.particle_types = tuple(ptypes)
             ptypes = self.dataset.particle_types
             ptypes_raw = self.dataset.particle_types_raw
         else:
@@ -481,6 +484,15 @@ class EnzoHierarchy(GridIndex):
             random_sample = np.mgrid[0:max(len(self.grids),1)].astype("int32")
         return self.grids[(random_sample,)]
 
+    def _get_particle_type_counts(self):
+        try:
+            ret = {}
+            for ptype in self.grid_active_particle_count:
+                ret[ptype] = self.grid_active_particle_count[ptype].sum()
+            return ret
+        except AttributeError:
+            return super(EnzoHierarchy, self)._get_particle_type_counts()
+
     def find_particles_by_type(self, ptype, max_num=None, additional_fields=None):
         """
         Returns a structure of arrays with all of the particles'
@@ -674,7 +686,8 @@ class EnzoDataset(Dataset):
                  parameter_override = None,
                  conversion_override = None,
                  storage_filename = None,
-                 units_override=None):
+                 units_override=None,
+                 unit_system="cgs"):
         """
         This class is a stripped down class that simply reads and parses
         *filename* without looking at the index.  *dataset_type* gets passed
@@ -692,7 +705,7 @@ class EnzoDataset(Dataset):
         self._conversion_override = conversion_override
         self.storage_filename = storage_filename
         Dataset.__init__(self, filename, dataset_type, file_style=file_style,
-                         units_override=units_override)
+                         units_override=units_override, unit_system=unit_system)
 
     def _setup_1d(self):
         self._index_class = EnzoHierarchy1D
@@ -905,11 +918,12 @@ class EnzoDataset(Dataset):
             if box_size is None:
                 box_size = self.parameters["Physics"]["Cosmology"]\
                     ["CosmologyComovingBoxSize"]
-            self.length_unit = self.quan(box_size, "Mpccm/h")
-            self.mass_unit = \
-                self.quan(k['urho'], 'g/cm**3') * (self.length_unit.in_cgs())**3
-            self.time_unit = self.quan(k['utim'], 's')
-            self.velocity_unit = self.quan(k['uvel'], 'cm/s')
+            setdefaultattr(self, 'length_unit', self.quan(box_size, "Mpccm/h"))
+            setdefaultattr(
+                self, 'mass_unit',
+                self.quan(k['urho'], 'g/cm**3') * (self.length_unit.in_cgs())**3)
+            setdefaultattr(self, 'time_unit', self.quan(k['utim'], 's'))
+            setdefaultattr(self, 'velocity_unit', self.quan(k['uvel'], 'cm/s'))
         else:
             if "LengthUnits" in self.parameters:
                 length_unit = self.parameters["LengthUnits"]
@@ -925,15 +939,16 @@ class EnzoDataset(Dataset):
                 mylog.warning("Setting 1.0 in code units to be 1.0 s")
                 length_unit = mass_unit = time_unit = 1.0
 
-            self.length_unit = self.quan(length_unit, "cm")
-            self.mass_unit = self.quan(mass_unit, "g")
-            self.time_unit = self.quan(time_unit, "s")
-            self.velocity_unit = self.length_unit / self.time_unit
+            setdefaultattr(self, 'length_unit', self.quan(length_unit, "cm"))
+            setdefaultattr(self, 'mass_unit', self.quan(mass_unit, "g"))
+            setdefaultattr(self, 'time_unit', self.quan(time_unit, "s"))
+            setdefaultattr(
+                self, 'velocity_unit', self.length_unit / self.time_unit)
 
         magnetic_unit = np.sqrt(4*np.pi * self.mass_unit /
                                 (self.time_unit**2 * self.length_unit))
         magnetic_unit = np.float64(magnetic_unit.in_cgs())
-        self.magnetic_unit = self.quan(magnetic_unit, "gauss")
+        setdefaultattr(self, 'magnetic_unit', self.quan(magnetic_unit, "gauss"))
 
     def cosmology_get_units(self):
         """
@@ -969,6 +984,14 @@ class EnzoDataset(Dataset):
             return True
         return os.path.exists("%s.hierarchy" % args[0])
 
+    @classmethod
+    def _guess_candidates(cls, base, directories, files):
+        candidates = [_ for _ in files if _.endswith(".hierarchy")
+                      and os.path.exists(
+                        os.path.join(base, _.rsplit(".", 1)[0]))]
+        # Typically, Enzo won't have nested outputs.
+        return candidates, (len(candidates) == 0)
+
 class EnzoDatasetInMemory(EnzoDataset):
     _index_class = EnzoHierarchyInMemory
     _dataset_type = 'enzo_inline'
@@ -1027,7 +1050,8 @@ class EnzoDatasetInMemory(EnzoDataset):
                 self.hubble_constant = self.cosmological_simulation = 0.0
 
     def _obtain_enzo(self):
-        import enzo; return enzo
+        import enzo
+        return enzo
 
     @classmethod
     def _is_valid(cls, *args, **kwargs):
