@@ -10,8 +10,6 @@ FITS-specific fields
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import numpy as np
-from yt.utilities.exceptions import *
 from yt.fields.field_info_container import \
     FieldInfoContainer
 
