@@ -14,10 +14,10 @@ HaloCatalog data-file handling function
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import h5py
+from yt.utilities.on_demand_imports import _h5py as h5py
 import numpy as np
 
-from yt.utilities.exceptions import *
+from yt.utilities.exceptions import YTDomainOverflow
 from yt.funcs import mylog
 
 from yt.utilities.io_handler import \
@@ -25,7 +25,6 @@ from yt.utilities.io_handler import \
 
 from yt.utilities.lib.geometry_utils import compute_morton
 
-from yt.geometry.oct_container import _ORDER_MAX
 
 class IOHandlerHaloCatalogHDF5(BaseIOHandler):
     _dataset_type = "halocatalog_hdf5"
@@ -44,7 +43,6 @@ class IOHandlerHaloCatalogHDF5(BaseIOHandler):
             for obj in chunk.objs:
                 data_files.update(obj.data_files)
         for data_file in sorted(data_files):
-            pcount = data_file.header['num_halos']
             with h5py.File(data_file.filename, "r") as f:
                 x = f['particle_position_x'].value.astype("float64")
                 y = f['particle_position_y'].value.astype("float64")
@@ -62,7 +60,6 @@ class IOHandlerHaloCatalogHDF5(BaseIOHandler):
             for obj in chunk.objs:
                 data_files.update(obj.data_files)
         for data_file in sorted(data_files):
-            pcount = data_file.header['num_halos']
             with h5py.File(data_file.filename, "r") as f:
                 for ptype, field_list in sorted(ptf.items()):
                     x = f['particle_position_x'].value.astype("float64")
