@@ -21,29 +21,30 @@ import stat
 import weakref
 from io import BytesIO
 
-from yt.funcs import *
+from yt.extern.six import string_types
+from yt.funcs import \
+    mylog, \
+    setdefaultattr
 from yt.geometry.oct_geometry_handler import \
     OctreeIndex
 from yt.geometry.geometry_handler import \
-    Index, YTDataChunk
+    YTDataChunk
 from yt.data_objects.static_output import \
     Dataset
 from yt.data_objects.octree_subset import \
     OctreeSubset
 
 from .definitions import ramses_header, field_aliases
-from yt.utilities.lib.misc_utilities import \
-    get_box_grids_level
-from yt.utilities.io_handler import \
-    io_registry
 from yt.utilities.physical_constants import mp, kb
 from .fields import \
     RAMSESFieldInfo, _X
 import yt.utilities.fortran_utils as fpu
 from yt.geometry.oct_container import \
     RAMSESOctreeContainer
-from yt.fields.particle_fields import \
-    standard_particle_fields
+from yt.arraytypes import blankRecordArray
+
+from yt.utilities.lib.cosmology_time import \
+    friedman
 
 class RAMSESDomainFile(object):
     _last_mask = None
@@ -108,7 +109,7 @@ class RAMSESDomainFile(object):
                     print("You are running with the wrong number of fields.")
                     print("If you specified these in the load command, check the array length.")
                     print("In this file there are %s hydro fields." % skipped)
-                    #print "The last set of field sizes was: %s" % skipped
+                    #print"The last set of field sizes was: %s" % skipped
                     raise
                 if hvals['file_ncache'] == 0: continue
                 assert(hvals['file_ilevel'] == level+1)
@@ -192,8 +193,8 @@ class RAMSESDomainFile(object):
             self.ngridbound = fpu.read_vector(f, 'i').astype("int64")
         else:
             self.ngridbound = np.zeros(hvals['nlevelmax'], dtype='int64')
-        free_mem = fpu.read_attrs(f, (('free_mem', 5, 'i'), ) )
-        ordering = fpu.read_vector(f, 'c')
+        free_mem = fpu.read_attrs(f, (('free_mem', 5, 'i'), ) )  # NOQA
+        ordering = fpu.read_vector(f, 'c')  # NOQA
         fpu.skip(f, 4)
         # Now we're at the tree itself
         # Now we iterate over each level and each CPU.
@@ -240,7 +241,7 @@ class RAMSESDomainFile(object):
                 #ng is the number of octs on this level on this domain
                 ng = _ng(cpu, level)
                 if ng == 0: continue
-                ind = fpu.read_vector(f, "I").astype("int64")
+                ind = fpu.read_vector(f, "I").astype("int64")  # NOQA
                 fpu.skip(f, 2)
                 pos = np.empty((ng, 3), dtype='float64')
                 pos[:,0] = fpu.read_vector(f, "d") - nx
@@ -464,6 +465,65 @@ class RAMSESIndex(OctreeIndex):
         for subset in oobjs:
             yield YTDataChunk(dobj, "io", [subset], None, cache = cache)
 
+    def _initialize_level_stats(self):
+        levels=sum([dom.level_count for dom in self.domains])
+        desc = {'names': ['numcells','level'],
+                'formats':['Int64']*2}
+        max_level=self.dataset.min_level+self.dataset.max_level+2
+        self.level_stats = blankRecordArray(desc, max_level)
+        self.level_stats['level'] = [i for i in range(max_level)]
+        self.level_stats['numcells'] = [0 for i in range(max_level)]
+        for level in range(self.dataset.min_level+1):
+            self.level_stats[level+1]['numcells']=2**(level*self.dataset.dimensionality)
+        for level in range(self.max_level+1):
+            self.level_stats[level+self.dataset.min_level+1]['numcells'] = levels[level]
+
+    def _get_particle_type_counts(self):
+        npart = 0
+        for dom in self.domains:
+            npart += dom.local_particle_count
+
+        return {'io': npart}
+
+    def print_stats(self):
+        
+        # This function prints information based on the fluid on the grids,
+        # and therefore does not work for DM only runs. 
+        if not self.fluid_field_list:
+            print("This function is not implemented for DM only runs")
+            return
+
+        self._initialize_level_stats()
+        """
+        Prints out (stdout) relevant information about the simulation
+        """
+        header = "%3s\t%14s\t%14s" % ("level", "# cells","# cells^3")
+        print(header)
+        print("%s" % (len(header.expandtabs())*"-"))
+        for level in range(self.dataset.min_level+self.dataset.max_level+2):
+            print("% 3i\t% 14i\t% 14i" % \
+                  (level,
+                   self.level_stats['numcells'][level],
+                   np.ceil(self.level_stats['numcells'][level]**(1./3))))
+        print("-" * 46)
+        print("   \t% 14i" % (self.level_stats['numcells'].sum()))
+        print("\n")
+
+        dx = self.get_smallest_dx()
+        try:
+            print("z = %0.8f" % (self.dataset.current_redshift))
+        except:
+            pass
+        print("t = %0.8e = %0.8e s = %0.8e years" % (
+            self.ds.current_time.in_units("code_time"),
+            self.ds.current_time.in_units("s"),
+            self.ds.current_time.in_units("yr")))
+        print("\nSmallest Cell:")
+        for item in ("Mpc", "pc", "AU", "cm"):
+            print("\tWidth: %0.3e %s" % (dx.in_units(item), item))
+
+
+
 class RAMSESDataset(Dataset):
     _index_class = RAMSESIndex
     _field_info_class = RAMSESFieldInfo
@@ -471,9 +531,9 @@ class RAMSESDataset(Dataset):
     
     def __init__(self, filename, dataset_type='ramses',
                  fields = None, storage_filename = None,
-                 units_override=None):
+                 units_override=None, unit_system="cgs"):
         # Here we want to initiate a traceback, if the reader is not built.
-        if isinstance(fields, str):
+        if isinstance(fields, string_types):
             fields = field_aliases[fields]
         '''
         fields: An array of hydro variable fields in order of position in the hydro_XXXXX.outYYYYY file
@@ -481,7 +541,8 @@ class RAMSESDataset(Dataset):
         '''
         self.fluid_types += ("ramses",)
         self._fields_in_file = fields
-        Dataset.__init__(self, filename, dataset_type, units_override=units_override)
+        Dataset.__init__(self, filename, dataset_type, units_override=units_override,
+                         unit_system=unit_system)
         self.storage_filename = storage_filename
 
     def __repr__(self):
@@ -508,17 +569,21 @@ class RAMSESDataset(Dataset):
         # For now assume an atomic ideal gas with cosmic abundances (x_H = 0.76)
         mean_molecular_weight_factor = _X**-1
 
-        self.density_unit = self.quan(density_unit, 'g/cm**3')
-        self.magnetic_unit = self.quan(magnetic_unit, "gauss")
-        self.pressure_unit = self.quan(pressure_unit, 'dyne/cm**2')
-        self.time_unit = self.quan(time_unit, "s")
-        self.mass_unit = self.quan(mass_unit, "g")
-        self.velocity_unit = self.quan(length_unit, 'cm') / self.time_unit
-        self.temperature_unit = (self.velocity_unit**2*mp* 
-                                 mean_molecular_weight_factor/kb).in_units('K')
+        setdefaultattr(self, 'density_unit', self.quan(density_unit, 'g/cm**3'))
+        setdefaultattr(self, 'magnetic_unit', self.quan(magnetic_unit, "gauss"))
+        setdefaultattr(self, 'pressure_unit',
+                       self.quan(pressure_unit, 'dyne/cm**2'))
+        setdefaultattr(self, 'time_unit', self.quan(time_unit, "s"))
+        setdefaultattr(self, 'mass_unit', self.quan(mass_unit, "g"))
+        setdefaultattr(self, 'velocity_unit',
+                       self.quan(length_unit, 'cm') / self.time_unit)
+        temperature_unit = (
+            self.velocity_unit**2*mp*mean_molecular_weight_factor/kb)
+        setdefaultattr(self, 'temperature_unit', temperature_unit.in_units('K'))
 
         # Only the length unit get scales by a factor of boxlen
-        self.length_unit = self.quan(length_unit * boxlen, "cm")
+        setdefaultattr(self, 'length_unit',
+                       self.quan(length_unit * boxlen, "cm"))
 
     def _parse_parameter_file(self):
         # hardcoded for now
@@ -558,7 +623,6 @@ class RAMSESDataset(Dataset):
                 dom, mi, ma = f.readline().split()
                 self.hilbert_indices[int(dom)] = (float(mi), float(ma))
         self.parameters.update(rheader)
-        self.current_time = self.parameters['time'] * self.parameters['unit_t']
         self.domain_left_edge = np.zeros(3, dtype='float64')
         self.domain_dimensions = np.ones(3, dtype='int32') * \
                         2**(self.min_level+1)
@@ -581,6 +645,23 @@ class RAMSESDataset(Dataset):
         self.max_level = rheader['levelmax'] - self.min_level - 1
         f.close()
 
+
+        if self.cosmological_simulation == 0:
+            self.current_time = self.parameters['time']
+        else :
+            self.tau_frw, self.t_frw, self.dtau, self.n_frw, self.time_tot = \
+                friedman( self.omega_matter, self.omega_lambda, 1. - self.omega_matter - self.omega_lambda )
+
+            age = self.parameters['time']
+            iage = 1 + int(10.*age/self.dtau)
+            iage = np.min([iage,self.n_frw//2 + (iage - self.n_frw//2)//10])
+
+            self.time_simu = self.t_frw[iage  ]*(age-self.tau_frw[iage-1])/(self.tau_frw[iage]-self.tau_frw[iage-1])+ \
+                             self.t_frw[iage-1]*(age-self.tau_frw[iage  ])/(self.tau_frw[iage-1]-self.tau_frw[iage])
+ 
+            self.current_time = (self.time_tot + self.time_simu)/(self.hubble_constant*1e7/3.08e24)/self.parameters['unit_t']
+
+
     @classmethod
     def _is_valid(self, *args, **kwargs):
         if not os.path.basename(args[0]).startswith("info_"): return False
