@@ -15,16 +15,44 @@ from __future__ import absolute_import
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
+import argparse
+import base64
+import getpass
+import numpy as np
+import os
+import sys
+import subprocess
+import tempfile
+import json
+import pprint
+
 from yt.config import ytcfg
 ytcfg["yt","__command_line"] = "True"
 from yt.startup_tasks import parser, subparsers
-from yt.mods import *
-from yt.funcs import *
-from yt.extern.six import add_metaclass
+from yt.funcs import \
+    ensure_list, \
+    get_hg_version, \
+    get_yt_version, \
+    mylog, \
+    ensure_dir_exists, \
+    update_hg, \
+    enable_plugins
+from yt.extern.six import add_metaclass, string_types
 from yt.extern.six.moves import urllib, input
-from yt.utilities.minimal_representation import MinimalProjectDescription
-import argparse, os, os.path, math, sys, time, subprocess, getpass, tempfile
-import base64, os
+from yt.convenience import load
+from yt.visualization.plot_window import \
+    SlicePlot, \
+    ProjectionPlot
+from yt.utilities.metadata import get_metadata
+from yt.utilities.exceptions import \
+    YTOutputNotIdentified, YTFieldNotParseable
+
+# loading field plugins for backward compatibility, since this module
+# used to do "from yt.mods import *"
+if ytcfg.getboolean("yt","loadfieldplugins"):
+    enable_plugins()
+
+_default_colormap = ytcfg.get("yt", "default_colormap")
 
 def _fix_ds(arg):
     if os.path.isdir("%s" % arg) and \
@@ -40,7 +68,7 @@ def _fix_ds(arg):
     return ds
 
 def _add_arg(sc, arg):
-    if isinstance(arg, str):
+    if isinstance(arg, string_types):
         arg = _common_options[arg].copy()
     argc = dict(arg.items())
     argnames = []
@@ -48,6 +76,48 @@ def _add_arg(sc, arg):
     if "longname" in argc: argnames.append(argc.pop('longname'))
     sc.add_argument(*argnames, **argc)
 
+def _print_failed_source_update(reinstall=False):
+    print()
+    print("The yt package is not installed from a mercurial repository,")
+    print("so you must update this installation manually.")
+    if 'Continuum Analytics' in sys.version or 'Anaconda' in sys.version:
+        # see http://stackoverflow.com/a/21318941/1382869 for why we need
+        # to check both Continuum *and* Anaconda
+        print()
+        print("Since it looks like you are using a python installation")
+        print("that is managed by conda, you may want to do:")
+        print()
+        print("    $ conda update yt")
+        print()
+        print("to update your yt installation.")
+        if reinstall is True:
+            print()
+            print("To update all of your packages, you can do:")
+            print()
+            print("    $ conda update --all")
+
+def _print_installation_information(path):
+    import yt
+    print()
+    print("yt module located at:")
+    print("    %s" % (path))
+    if "YT_DEST" in os.environ:
+        spath = os.path.join(
+            os.environ["YT_DEST"], "src", "yt-supplemental")
+        if os.path.isdir(spath):
+            print("The supplemental repositories are located at:")
+            print("    %s" % (spath))
+    print()
+    print("The current version of yt is:")
+    print()
+    print("---")
+    print("Version = %s" % yt.__version__)
+    vstring = get_hg_version(path)
+    if vstring is not None:
+        print("Changeset = %s" % vstring.strip().decode("utf-8"))
+    print("---")
+    return vstring
+
 class YTCommandSubtype(type):
     def __init__(cls, name, b, d):
         type.__init__(cls, name, b, d)
@@ -72,6 +142,16 @@ class YTCommand(object):
     @classmethod
     def run(cls, args):
         self = cls()
+        # Check for some things we know; for instance, comma separated
+        # field names should be parsed as tuples.
+        if getattr(args, 'field', None) is not None and ',' in args.field:
+            if args.field.count(",") > 1:
+                raise YTFieldNotParseable(args.field)
+            args.field = tuple(_.strip() for _ in args.field.split(","))
+        if getattr(args, 'weight', None) is not None and ',' in args.weight:
+            if args.weight.count(",") > 1:
+                raise YTFieldNotParseable(args.weight)
+            args.weight = tuple(_.strip() for _ in args.weight.split(","))
         # Some commands need to be run repeatedly on datasets
         # In fact, this is the rule and the opposite is the exception
         # BUT, we only want to parse the arguments once.
@@ -129,14 +209,16 @@ _common_options = dict(
     field   = dict(short="-f", longname="--field",
                    action="store", type=str,
                    dest="field", default="density",
-                   help="Field to color by"),
+                   help=("Field to color by, "
+                         "use a comma to separate field tuple values")),
     weight  = dict(short="-g", longname="--weight",
                    action="store", type=str,
                    dest="weight", default=None,
-                   help="Field to weight projections with"),
+                   help=("Field to weight projections with, "
+                         "use a comma to separate field tuple values")),
     cmap    = dict(longname="--colormap",
                    action="store", type=str,
-                   dest="cmap", default="algae",
+                   dest="cmap", default=_default_colormap,
                    help="Colormap name"),
     zlim    = dict(short="-z", longname="--zlim",
                    action="store", type=float,
@@ -308,6 +390,13 @@ def _get_yt_stack_date():
 def _update_yt_stack(path):
     "Rerun the install script to updated all dependencies."
 
+    if "YT_DEST" not in os.environ:
+        print()
+        print("This yt installation does not appear to be managed by the")
+        print("source-based install script, but 'update --all' was specified.")
+        print("You will need to update your dependencies manually.")
+        return
+
     install_script = os.path.join(path, "doc/install_script.sh")
     if not os.path.exists(install_script):
         print()
@@ -335,19 +424,6 @@ def _update_yt_stack(path):
         print("The yt stack has been updated successfully.")
         print("Now get back to work!")
 
-def get_yt_version():
-    try:
-        from yt.__hg_version__ import hg_version
-        return hg_version
-    except ImportError:
-        pass
-    import pkg_resources
-    yt_provider = pkg_resources.get_provider("yt")
-    path = os.path.dirname(yt_provider.module_path)
-    if not os.path.isdir(os.path.join(path, ".hg")): return None
-    version = get_hg_version(path)
-    return version
-
 # This code snippet is modified from Georg Brandl
 def bb_apicall(endpoint, data, use_pass = True):
     uri = 'https://api.bitbucket.org/1.0/%s/' % endpoint
@@ -423,7 +499,7 @@ class YTBugreportCmd(YTCommand):
             tf = tempfile.NamedTemporaryFile(delete=False)
             fn = tf.name
             tf.close()
-            popen = subprocess.call("$EDITOR %s" % fn, shell = True)
+            subprocess.call("$EDITOR %s" % fn, shell = True)
             content = open(fn).read()
             try:
                 os.unlink(fn)
@@ -544,7 +620,7 @@ class YTHubRegisterCmd(YTCommand):
         hub_url = "https://hub.yt-project.org/create_user"
         req = urllib.request.Request(hub_url, data)
         try:
-            status = urllib.request.urlopen(req).read()
+            urllib.request.urlopen(req).read()
         except urllib.error.HTTPError as exc:
             if exc.code == 400:
                 print("Sorry, the Hub couldn't create your user.")
@@ -579,45 +655,17 @@ class YTInstInfoCmd(YTCommand):
 
     def __call__(self, opts):
         import pkg_resources
-        import yt
         yt_provider = pkg_resources.get_provider("yt")
         path = os.path.dirname(yt_provider.module_path)
-        print()
-        print("yt module located at:")
-        print("    %s" % (path))
-        update_supp = False
-        if "YT_DEST" in os.environ:
-            spath = os.path.join(
-                     os.environ["YT_DEST"], "src", "yt-supplemental")
-            if os.path.isdir(spath):
-                print("The supplemental repositories are located at:")
-                print("    %s" % (spath))
-                update_supp = True
-        vstring = get_yt_version()
-        if vstring == -1:
-            vstring = "unknown"
+        vstring = _print_installation_information(path)
         if vstring is not None:
-            print()
-            print("The current version and changeset for the code is:")
-            print()
-            print("---")
-            print("Version = %s" % yt.__version__)
-            print("Changeset = %s" % vstring.strip().decode("utf-8"))
-            print("---")
-            print()
-            if "site-packages" not in path:
-                print("This installation CAN be automatically updated.")
-                if opts.update_source:
-                    update_hg(path)
-                    print("Updated successfully.")
+            print("This installation CAN be automatically updated.")
+            if opts.update_source:
+                update_hg(path)
+                print("Updated successfully.")
                 _get_yt_stack_date()
         elif opts.update_source:
-            print()
-            print("YT site-packages not in path, so you must")
-            print("update this installation manually by committing and")
-            print("merging your modifications to the code before")
-            print("updating to the newest changeset.")
-            print()
+            _print_failed_source_update()
         if vstring is not None and opts.outputfile is not None:
             open(opts.outputfile, "w").write(vstring)
 
@@ -657,7 +705,10 @@ class YTLoadCmd(YTCommand):
                   "\nHi there!  Welcome to yt.\n\nWe've loaded your dataset as 'ds'.  Enjoy!"
                   )
         else:
-            from IPython.config.loader import Config
+            try:
+                from traitlets.config.loader import Config
+            except ImportError:
+                from IPython.config.loader import Config
             import sys
             cfg = Config()
             # prepend sys.path with current working directory
@@ -692,10 +743,12 @@ class YTMapserverCmd(YTCommand):
             p = ProjectionPlot(ds, args.axis, args.field, weight_field=args.weight)
         else:
             p = SlicePlot(ds, args.axis, args.field)
-        from yt.gui.reason.pannable_map import PannableMapServer
-        mapper = PannableMapServer(p.data_source, args.field)
+        from yt.visualization.mapserver.pannable_map import PannableMapServer
+        PannableMapServer(p.data_source, args.field)
         import yt.extern.bottle as bottle
         bottle.debug(True)
+        bottle_dir = os.path.dirname(bottle.__file__)
+        sys.path.append(bottle_dir)
         if args.host is not None:
             colonpl = args.host.find(":")
             if colonpl >= 0:
@@ -706,6 +759,7 @@ class YTMapserverCmd(YTCommand):
             bottle.run(server='rocket', host=args.host, port=port)
         else:
             bottle.run(server='rocket')
+        sys.path.remove(bottle_dir)
 
 
 class YTPastebinCmd(YTCommand):
@@ -967,11 +1021,11 @@ class YTStatsCmd(YTCommand):
         ds.print_stats()
         vals = {}
         if args.field in ds.derived_field_list:
-            if args.max == True:
+            if args.max is True:
                 vals['min'] = ds.find_max(args.field)
                 print("Maximum %s: %0.5e at %s" % (args.field,
                     vals['min'][0], vals['min'][1]))
-            if args.min == True:
+            if args.min is True:
                 vals['max'] = ds.find_min(args.field)
                 print("Minimum %s: %0.5e at %s" % (args.field,
                     vals['max'][0], vals['max'][1]))
@@ -997,30 +1051,10 @@ class YTUpdateCmd(YTCommand):
 
     def __call__(self, opts):
         import pkg_resources
-        import yt
         yt_provider = pkg_resources.get_provider("yt")
         path = os.path.dirname(yt_provider.module_path)
-        print()
-        print("yt module located at:")
-        print("    %s" % (path))
-        update_supp = False
-        if "YT_DEST" in os.environ:
-            spath = os.path.join(
-                     os.environ["YT_DEST"], "src", "yt-supplemental")
-            if os.path.isdir(spath):
-                print("The supplemental repositories are located at:")
-                print("    %s" % (spath))
-                update_supp = True
-        vstring = None
-        if "site-packages" not in path:
-            vstring = get_hg_version(path)
-            print()
-            print("The current version and changeset for the code is:")
-            print()
-            print("---")
-            print("Version = %s" % yt.__version__)
-            print("Changeset = %s" % vstring.strip().decode("utf-8"))
-            print("---")
+        vstring = _print_installation_information(path)
+        if vstring is not None:
             print()
             print("This installation CAN be automatically updated.")
             update_hg(path, skip_rebuild=opts.reinstall)
@@ -1028,13 +1062,41 @@ class YTUpdateCmd(YTCommand):
             _get_yt_stack_date()
             if opts.reinstall:
                 _update_yt_stack(path)
+        else:
+            _print_failed_source_update(opts.reinstall)
+
+
+class YTDeleteImageCmd(YTCommand):
+    args = (dict(short="delete_hash", type=str),)
+    description = \
+        """
+        Delete image from imgur.com.
+
+        """
+    name = "delete_image"
+    def __call__(self, args):
+        headers = {'Authorization':
+            'Client-ID {}'.format(ytcfg.get("yt", "imagebin_api_key"))}
+
+        delete_url = ytcfg.get("yt", "imagebin_delete_url")
+        req = urllib.request.Request(
+            delete_url.format(delete_hash=args.delete_hash),
+            headers=headers, method='DELETE')
+        try:
+            response = urllib.request.urlopen(req).read().decode()
+        except urllib.error.HTTPError as e:
+            print("ERROR", e)
+            return {'deleted': False}
+
+        rv = json.loads(response)
+        if 'success' in rv and rv["success"]:
+            print("\nImage successfully deleted!\n")
         else:
             print()
-            print("YT site-packages not in path, so you must")
-            print("update this installation manually by committing and")
-            print("merging your modifications to the code before")
-            print("updating to the newest changeset.")
+            print("Something has gone wrong!  Here is the server response:")
             print()
+            pprint.pprint(rv)
+
 
 class YTUploadImageCmd(YTCommand):
     args = (dict(short="file", type=str),)
@@ -1049,27 +1111,29 @@ class YTUploadImageCmd(YTCommand):
         if not filename.endswith(".png"):
             print("File must be a PNG file!")
             return 1
-        import base64, json, pprint
+        headers = {'Authorization':
+            'Client-ID {}'.format(ytcfg.get("yt", "imagebin_api_key"))}
+
         image_data = base64.b64encode(open(filename, 'rb').read())
-        api_key = 'f62d550859558f28c4c214136bc797c7'
-        parameters = {'key':api_key, 'image':image_data, type:'base64',
-                      'caption': "",
+        parameters = {'image': image_data, type: 'base64',
+                      'name': filename,
                       'title': "%s uploaded by yt" % filename}
         data = urllib.parse.urlencode(parameters).encode('utf-8')
-        req = urllib.request.Request('http://api.imgur.com/2/upload.json', data)
+        req = urllib.request.Request(
+            ytcfg.get("yt", "imagebin_upload_url"), data=data, headers=headers)
         try:
             response = urllib.request.urlopen(req).read().decode()
         except urllib.error.HTTPError as e:
             print("ERROR", e)
             return {'uploaded':False}
         rv = json.loads(response)
-        if 'upload' in rv and 'links' in rv['upload']:
+        if 'data' in rv and 'link' in rv['data']:
             print()
             print("Image successfully uploaded!  You can find it at:")
-            print("    %s" % (rv['upload']['links']['original']))
+            print("    %s" % (rv['data']['link']))
             print()
-            print("If you'd like to delete it, visit this page:")
-            print("    %s" % (rv['upload']['links']['delete_page']))
+            print("If you'd like to delete it, use the following")
+            print("    yt delete_image %s" % rv['data']['deletehash'])
             print()
         else:
             print()
@@ -1077,6 +1141,54 @@ class YTUploadImageCmd(YTCommand):
             print()
             pprint.pprint(rv)
 
+class YTSearchCmd(YTCommand):
+    args = (dict(short="-o", longname="--output",
+                 action="store", type=str,
+                 dest="output", default="yt_index.json",
+                 help="File in which to place output"),
+            dict(longname="--check-all", short="-a",
+                 help="Attempt to load every file",
+                 action="store_true", default=False,
+                 dest="check_all"),
+            dict(longname="--full", short="-f",
+                 help="Output full contents of parameter file",
+                 action="store_true", default=False,
+                 dest="full_output"),
+            )
+    description = \
+        """
+        Attempt to find outputs that yt can recognize in directories.
+        """
+    name = "search"
+    def __call__(self, args):
+        from yt.utilities.parameter_file_storage import \
+            output_type_registry
+        candidates = []
+        for base, dirs, files in os.walk(".", followlinks=True):
+            print("(% 10i candidates) Examining %s" % (len(candidates), base))
+            recurse = []
+            if args.check_all:
+                candidates.extend([os.path.join(base, _) for _ in files])
+            for _, otr in sorted(output_type_registry.items()):
+                c, r = otr._guess_candidates(base, dirs, files)
+                candidates.extend([os.path.join(base, _) for _ in c])
+                recurse.append(r)
+            if len(recurse) > 0 and not all(recurse):
+                del dirs[:]
+        # Now we have a ton of candidates.  We're going to do something crazy
+        # and try to load each one.
+        records = []
+        for i, c in enumerate(sorted(candidates)):
+            print("(% 10i/% 10i) Evaluating %s" % (i, len(candidates), c))
+            try:
+                record = get_metadata(c, args.full_output)
+            except YTOutputNotIdentified:
+                continue
+            records.append(record)
+        with open(args.output, "w") as f:
+            json.dump(records, f, indent=4)
+        print("Identified %s records output to %s" % (
+              len(records), args.output))
 
 def run_main():
     args = parser.parse_args()
