@@ -13,9 +13,7 @@ A wrapper class for h5py file objects.
 # The full license is in the file COPYING.txt, distributed with this software.
 #-----------------------------------------------------------------------------
 
-import h5py
-
-from distutils.version import LooseVersion
+from yt.utilities.on_demand_imports import _h5py as h5py
 
 class HDF5FileHandler(object):
     handle = None
@@ -23,15 +21,6 @@ class HDF5FileHandler(object):
     def __init__(self, filename):
         self.handle = h5py.File(filename, 'r')
 
-    def __del__(self):
-        if self.handle is not None:
-            # In h5py 2.4 and newer file handles are closed automatically.
-            # so only close the handle on old versions.  This works around an
-            # issue in h5py when run under python 3.4.
-            # See https://github.com/h5py/h5py/issues/534
-            if LooseVersion(h5py.__version__) < '2.4.0':
-                self.handle.close()
-
     def __getitem__(self, key):
         return self.handle[key]
 
@@ -51,6 +40,10 @@ class HDF5FileHandler(object):
     def items(self):
         return list(self.handle.items())
 
+    def close(self):
+        if self.handle is not None:
+            self.handle.close()
+
 class FITSFileHandler(HDF5FileHandler):
     def __init__(self, filename):
         from yt.utilities.on_demand_imports import _astropy
@@ -70,7 +63,12 @@ class FITSFileHandler(HDF5FileHandler):
         del self._fits_files
         del self.handle
         self.handle = None
-        super(FITSFileHandler, self).__del__()
 
     def close(self):
-        pass
+        self.handle.close()
+
+class NetCDF4FileHandler(object):
+    def __init__(self, filename):
+        from netCDF4 import Dataset
+        ds = Dataset(filename)
+        self.dataset = ds
