@@ -1,15 +1,19 @@
 from __future__ import print_function
 from yt.extern.six.moves import cStringIO
-import re
 import os
 import numpy as np
-try:
-    from thingking.httpmmap import HTTPArray
-    from thingking.arbitrary_page import PageCacheURL
-except ImportError:
-    HTTPArray = PageCacheURL = None
+
 from yt.funcs import mylog
 
+def get_thingking_deps():
+    try:
+        from thingking.httpmmap import HTTPArray
+        from thingking.arbitrary_page import PageCacheURL
+    except ImportError:
+        raise ImportError(
+            "This functionality requires the thingking package to be installed")
+    return HTTPArray, PageCacheURL
+
 _types = {
     'int16_t': 'int16',
     'uint16_t': 'uint16',
@@ -217,8 +221,8 @@ class HTTPDataStruct(DataStruct):
 
     def __init__(self, *args, **kwargs):
         super(HTTPDataStruct, self).__init__(*args, **kwargs)
-        if None in (PageCacheURL, HTTPArray):
-            raise ImportError("'thingking' is required for loading of remote HTTP data.")
+        HTTPArray, PageCacheURL = get_thingking_deps()
+        self.HTTPArray = HTTPArray
         self.pcu = PageCacheURL(self.filename)
 
     def set_offset(self, offset):
@@ -233,7 +237,7 @@ class HTTPDataStruct(DataStruct):
     def build_memmap(self):
         assert(self.size != -1)
         mylog.info('Building memmap with offset: %i and size %i' % (self._offset, self.size))
-        self.handle = HTTPArray(self.filename, dtype=self.dtype,
+        self.handle = self.HTTPArray(self.filename, dtype=self.dtype,
                         shape=self.size, offset=self._offset)
         for k in self.dtype.names:
             self.data[k] = RedirectArray(self.handle, k)
@@ -387,8 +391,6 @@ class SDFRead(dict):
         assert 'struct' in line
 
         str_types = []
-        comments = []
-        str_lines = []
         l = ascfile.readline()
         while "}" not in l:
             vtype, vnames = _get_struct_vars(l)
@@ -460,14 +462,14 @@ class HTTPSDFRead(SDFRead):
     _data_struct = HTTPDataStruct
 
     def __init__(self, *args, **kwargs):
-        if None in (PageCacheURL, HTTPArray):
-            raise ImportError("thingking")
+        HTTPArray, _ = get_thingking_deps()
+        self.HTTPArray = HTTPArray
         super(HTTPSDFRead, self).__init__(*args, **kwargs)
 
     def parse_header(self):
         """docstring for parse_header"""
         # Pre-process
-        ascfile = HTTPArray(self.header)
+        ascfile = self.HTTPArray(self.header)
         max_header_size = 1024*1024
         lines = cStringIO(ascfile[:max_header_size].data[:])
         while True:
@@ -1036,10 +1038,7 @@ class SDFIndex(object):
         #right = right.astype('float32')
 
         #my_filter = bbox_filter(left, right, self.true_domain_width)
-        data = []
-        for dd in self.filter_bbox(
-            left, right,
-            self.iter_data(inds, fields)):
+        for dd in self.filter_bbox(left, right, self.iter_data(inds, fields)):
             yield dd
         #for dd in self.filter_particles(
         #    self.iter_data(inds, fields),
@@ -1066,11 +1065,8 @@ class SDFIndex(object):
         return self.iter_data(inds, fields)
 
     def get_contiguous_chunk(self, left_key, right_key, fields):
-        liarr = self.get_ind_from_key(left_key)
-        riarr = self.get_ind_from_key(right_key)
 
         lbase=0
-        llen = 0
         if left_key > self._max_key:
             raise RuntimeError("Left key is too large. Key: %i Max Key: %i" % \
                                (left_key, self._max_key))
@@ -1080,7 +1076,6 @@ class SDFIndex(object):
         right_key = self.get_previous_nonzero_chunk(right_key, left_key)
 
         lbase = self.indexdata['base'][left_key]
-        llen = self.indexdata['len'][left_key]
 
         rbase = self.indexdata['base'][right_key]
         rlen = self.indexdata['len'][right_key]
@@ -1247,8 +1242,7 @@ class SDFIndex(object):
 
         pbox[1, 0] = bbox[1, 1]
         pbox[1, 1] = pbox[1, 0] + pad[1]
-        for dd in self.filter_bbox(
-            filter_left, filter_right,
+        for dd in self.filter_bbox(filter_left, filter_right,
             self.iter_bbox_data(pbox[:,0], pbox[:,1], fields)):
             yield dd
             del dd
@@ -1284,9 +1278,6 @@ class SDFIndex(object):
 
         """
         _ensure_xyz_fields(fields)
-        bbox = self.get_cell_bbox(level, cell_iarr)
-        filter_left = bbox[:, 0] - pad
-        filter_right = bbox[:, 1] + pad
 
         data = []
         for dd in self.iter_padded_bbox_data(level, cell_iarr, pad, fields):
@@ -1304,8 +1295,6 @@ class SDFIndex(object):
 
         """
         bbox = self.get_cell_bbox(level, cell_iarr)
-        filter_left = bbox[:, 0] - pad
-        filter_right = bbox[:, 1] + pad
 
         # Need to get all of these
         low_key, high_key = self.get_key_bounds(level, cell_iarr)
