@@ -1,4 +1,8 @@
-from yt.testing import *
+import numpy as np
+
+from yt.testing import \
+    fake_random_ds, \
+    assert_almost_equal
 from yt.utilities.math_utils import euclidean_dist, periodic_dist
 
 def setup():
