@@ -17,9 +17,11 @@ from __future__ import absolute_import
 import numpy as np
 from matplotlib.cm import get_cmap
 
-from yt.funcs import *
+from yt.funcs import \
+    mylog, ensure_list
 
-from yt.utilities.physical_constants import *
+from yt.utilities.physical_constants import \
+    clight, hcgs, kboltz
 
 class TransferFunction(object):
     r"""A transfer function governs the transmission of emission and
@@ -27,7 +29,7 @@ class TransferFunction(object):
 
     Transfer functions are defined by boundaries, bins, and the value that
     governs transmission through that bin.  This is scaled between 0 and 1.
-    When integrating through a volume. the value through a given cell is
+    When integrating through a volume the value through a given cell is
     defined by the value calculated in the transfer function.
 
     Parameters
@@ -36,7 +38,7 @@ class TransferFunction(object):
         The min and max for the transfer function.  Values below or above
         these values are discarded.
     nbins : int
-        How many bins to calculate; in betwee, linear interpolation is
+        How many bins to calculate; in between, linear interpolation is
         used, so low values are typically fine.
 
     Notes
@@ -56,11 +58,12 @@ class TransferFunction(object):
         self.y = np.zeros(nbins, dtype='float64')
         self.grad_field = -1
         self.light_source_v = self.light_source_c = np.zeros(3, 'float64')
+        self.features = []
 
     def add_gaussian(self, location, width, height):
         r"""Add a Gaussian distribution to the transfer function.
 
-        Typically, when rendering isocontours, a Guassian distribution is the
+        Typically, when rendering isocontours, a Gaussian distribution is the
         easiest way to draw out features.  The spread provides a softness.
         The values are calculated as :math:`f(x) = h \exp{-(x-x_0)^2 / w}`.
 
@@ -83,6 +86,8 @@ class TransferFunction(object):
         """
         vals = height * np.exp(-(self.x - location)**2.0/width)
         self.y = np.clip(np.maximum(vals, self.y), 0.0, np.inf)
+        self.features.append(('gaussian', "location(x):%3.2g" % location, 
+                              "width(x):%3.2g" % width, "height(y):%3.2g" % height))
 
     def add_line(self, start, stop):
         r"""Add a line between two points to the transmission function.
@@ -116,6 +121,9 @@ class TransferFunction(object):
         vals = slope * (self.x - x0) + y0
         vals[~((self.x >= x0) & (self.x <= x1))] = 0.0
         self.y = np.clip(np.maximum(vals, self.y), 0.0, np.inf)
+        self.features.append(('line', "start(x,y):(%3.2g, %3.2g)" % \
+                              (start[0], start[1]), "stop(x,y):(%3.2g, %3.2g)"\
+                              % (stop[0], stop[1])))
 
     def add_step(self, start, stop, value):
         r"""Adds a step function to the transfer function.
@@ -150,6 +158,8 @@ class TransferFunction(object):
         vals = np.zeros(self.x.shape, 'float64')
         vals[(self.x >= start) & (self.x <= stop)] = value
         self.y = np.clip(np.maximum(vals, self.y), 0.0, np.inf)
+        self.features.append(('step', "start(x):%3.2g" % start, \
+                              "stop(x):%3.2g" % stop, "value(y):%3.2g" % value))
 
     def add_filtered_planck(self, wavelength, trans):
         vals = np.zeros(self.x.shape, 'float64')
@@ -187,7 +197,9 @@ class TransferFunction(object):
         >>> tf.add_gaussian(-9.0, 0.01, 1.0)
         >>> tf.plot("sample.png")
         """
-        import matplotlib;matplotlib.use("Agg");import pylab
+        import matplotlib
+        matplotlib.use("Agg")
+        import pylab
         pylab.clf()
         pylab.plot(self.x, self.y, 'xk-')
         pylab.xlim(*self.x_bounds)
@@ -209,7 +221,7 @@ class TransferFunction(object):
         >>> tf.add_gaussian(-9.0, 0.01, 1.0)
         >>> tf.show()
         """
-        import matplotlib;import pylab
+        import pylab
         pylab.clf()
         pylab.plot(self.x, self.y, 'xk-')
         pylab.xlim(*self.x_bounds)
@@ -218,10 +230,16 @@ class TransferFunction(object):
         
     def clear(self):
         self.y[:]=0.0
+        self.features = []
+
+    def __repr__(self):
+        disp = "<Transfer Function Object>: x_bounds:(%3.2g, %3.2g) nbins:%3.2g features:%s" % \
+                (self.x_bounds[0], self.x_bounds[1], self.nbins, self.features)
+        return disp
 
 class MultiVariateTransferFunction(object):
     r"""This object constructs a set of field tables that allow for
-    multiple field variables to control the integration through a volme.
+    multiple field variables to control the integration through a volume.
 
     The integration through a volume typically only utilizes a single field
     variable (for instance, Density) to set up and control the values
@@ -348,7 +366,7 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         The min and max for the transfer function.  Values below or above
         these values are discarded.
     nbins : int
-        How many bins to calculate; in betwee, linear interpolation is
+        How many bins to calculate; in between, linear interpolation is
         used, so low values are typically fine.
     grey_opacity : bool
         Should opacity be calculated on a channel-by-channel basis, or
@@ -367,6 +385,7 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         self.alpha = TransferFunction(x_bounds, nbins)
         self.funcs = (self.red, self.green, self.blue, self.alpha)
         self.grey_opacity = grey_opacity
+        self.features = []
 
         # Now we do the multivariate stuff
         # We assign to Density, but do not weight
@@ -406,6 +425,10 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         """
         for tf, v in zip(self.funcs, height):
             tf.add_gaussian(location, width, v)
+        self.features.append(('gaussian', "location(x):%3.2g" % location, \
+                              "width(x):%3.2g" % width, \
+                              "height(y):(%3.2g, %3.2g, %3.2g, %3.2g)" % 
+                              (height[0], height[1], height[2], height[3])))
 
     def add_step(self, start, stop, value):
         r"""Adds a step function to the transfer function.
@@ -439,6 +462,10 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         """
         for tf, v in zip(self.funcs, value):
             tf.add_step(start, stop, v)
+        self.features.append(('step', "start(x):%3.2g" % start, \
+                              "stop(x):%3.2g" % stop, \
+                              "value(y):(%3.2g, %3.2g, %3.2g, %3.2g)" % \
+                              (value[0], value[1], value[2], value[3])))
 
     def plot(self, filename):
         r"""Save an image file of the transfer function.
@@ -527,7 +554,8 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         ax.set_ylabel("Opacity")
         ax.set_xlabel("Value")
 
-    def vert_cbar(self, ax=None, label=None, label_fmt=None):
+    def vert_cbar(self, resolution, log_scale, ax=None, label=None, 
+                  label_fmt=None):
         r"""Display an image of the transfer function
 
         This function loads up matplotlib and displays the current transfer function.
@@ -576,10 +604,15 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         ax.yaxis.set_ticks(xticks)
         def x_format(x, pos):
             val = x * (self.alpha.x[-1] - self.alpha.x[0]) / (self.alpha.x.size-1) + self.alpha.x[0]
-            if label_fmt == None:
+            if log_scale is True:
+                val = 10**val
+            if label_fmt is None:
                 if abs(val) < 1.e-3 or abs(val) > 1.e4:
-                    e = np.floor(np.log10(abs(val)))
-                    return r"${:.2f}\times 10^{:d}$".format(val/10.0**e, int(e))
+                    if not val == 0.0:
+                        e = np.floor(np.log10(abs(val)))
+                        return r"${:.2f}\times 10^{:d}$".format(val/10.0**e, int(e))
+                    else:
+                        return r"$0$"
                 else:
                     return "%.1g" % (val)
             else:
@@ -596,7 +629,7 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         ax.get_xaxis().set_ticks([])
         ax.set_ylim(visible[0], visible[-1])
         ax.tick_params(axis='y', colors='white', size=10)
-        ax.set_ylabel(label, color='white')
+        ax.set_ylabel(label, color='white', size=10*resolution/512.0)
         
 
         
@@ -634,7 +667,7 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         --------
 
         >>> tf = ColorTransferFunction( (-10.0, -5.0) )
-        >>> tf.sample_colormap(-7.0, 0.01, colormap='algae')
+        >>> tf.sample_colormap(-7.0, 0.01, colormap='arbre')
         """
         v = np.float64(v)
         if col_bounds is None:
@@ -678,10 +711,10 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         --------
 
         >>> def linramp(vals, minval, maxval):
-        ...     return (vals - vals.min())/(vals.(max) - vals.min())
+        ...     return (vals - vals.min())/(vals.max() - vals.min())
         >>> tf = ColorTransferFunction( (-10.0, -5.0) )
-        >>> tf.map_to_colormap(-8.0, -6.0, scale=10.0, colormap='algae')
-        >>> tf.map_to_colormap(-6.0, -5.0, scale=10.0, colormap='algae',
+        >>> tf.map_to_colormap(-8.0, -6.0, scale=10.0, colormap='arbre')
+        >>> tf.map_to_colormap(-6.0, -5.0, scale=10.0, colormap='arbre',
         ...                    scale_func = linramp)
         """
         mi = np.float64(mi)
@@ -703,6 +736,9 @@ class ColorTransferFunction(MultiVariateTransferFunction):
         self.green.y[rel0:rel1] = cc[:, 1]*scale_mult
         self.blue.y[rel0:rel1] = cc[:, 2]*scale_mult
         self.alpha.y[rel0:rel1] = scale*cc[:, 3]*scale_mult
+        self.features.append(('map_to_colormap', "start(x):%3.2g" % mi, \
+                              "stop(x):%3.2g" % ma, \
+                              "value(y):%3.2g" % scale))
 
     def add_layers(self, N, w=None, mi=None, ma=None, alpha = None,
                    colormap="gist_stern", col_bounds = None):
@@ -756,7 +792,9 @@ class ColorTransferFunction(MultiVariateTransferFunction):
             dist = (col_bounds[1] - col_bounds[0])
             if mi is None: mi = col_bounds[0] + dist/(10.0*N)
             if ma is None: ma = col_bounds[1] - dist/(10.0*N)
-        if w is None: w = 0.001 * (ma-mi)/N
+        if w is None:
+            w = 0.001 * (ma - mi) / N
+            w = max(w, 1.0 / self.nbins)
         if alpha is None and self.grey_opacity:
             alpha = np.ones(N, dtype="float64")
         elif alpha is None and not self.grey_opacity:
@@ -776,7 +814,15 @@ class ColorTransferFunction(MultiVariateTransferFunction):
     def clear(self):
         for f in self.funcs:
             f.clear()
+        self.features = []
 
+    def __repr__(self):
+        disp = "<Color Transfer Function Object>:\n" + \
+                "x_bounds:[%3.2g, %3.2g] nbins:%i features:\n" % (self.x_bounds[0],
+                        self.x_bounds[1], self.nbins)
+        for f in self.features:
+            disp += "\t%s\n" % str(f)
+        return disp
 
 class ProjectionTransferFunction(MultiVariateTransferFunction):
     r"""A transfer function that defines a simple projection.
@@ -785,7 +831,7 @@ class ProjectionTransferFunction(MultiVariateTransferFunction):
     this transfer function should be used.  It will create a very simple
     table that merely sums along each ray.  Note that the end product will
     need to be scaled by the total width through which the rays were cast,
-    a piece of information inacessible to the transfer function.
+    a piece of information inaccessible to the transfer function.
 
     Parameters
     ----------
@@ -849,7 +895,8 @@ class PlanckTransferFunction(MultiVariateTransferFunction):
             scat = (johnson_filters[f]["Lchar"]**-4 / mscat)*anorm
             tf = TransferFunction(rho_bounds)
             mylog.debug("Adding: %s with relative scattering %s" % (f, scat))
-            tf.y *= 0.0; tf.y += scat
+            tf.y *= 0.0
+            tf.y += scat
             self.add_field_table(tf, 1, weight_field_id = 1)
             self.link_channels(i+3, i+3)
 
